@@ -39,16 +39,20 @@
 
 #include "config.h"
 
+#define _GNU_SOURCE /* For clone */
 #include <fcntl.h>
+#include <limits.h>
 #include <poll.h>
+#include <sched.h>
 #include <signal.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/mman.h>
+#include <sys/mount.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <sys/param.h>		/* MAXPATHLEN */
 #include <unistd.h>
 
 #include "slurm/slurm.h"
@@ -65,6 +69,7 @@
 #include "src/common/slurm_protocol_defs.h"
 #include "src/common/slurm_step_layout.h"
 #include "src/common/slurmdb_defs.h"
+#include "src/common/spank.h"
 #include "src/common/strlcpy.h"
 #include "src/common/xassert.h"
 #include "src/common/xmalloc.h"
@@ -92,6 +97,18 @@ strong_alias(env_unset_environment,	slurm_env_unset_environment);
 #define MAX_ENV_STRLEN (32 * 4096)	/* Needed for CPU_BIND and MEM_BIND on
 					 * SGI systems with huge CPU counts */
 
+#define STACK_SIZE (1024 * 1024) /* For clone() syscall. */
+
+typedef struct {
+	char *cmdstr;
+	int *fildes;
+	int mode;
+	int rlimit;
+	char **tmp_env;
+	const char *username;
+	bool setup_namespaces;
+} child_args_t;
+
 /*
  *  Return pointer to `name' entry in environment if found, or
  *   pointer to the last entry (i.e. NULL) if `name' is not
@@ -286,7 +303,7 @@ char *getenvp(char **env, const char *name)
 int setup_env(env_t *env, bool preserve_env)
 {
 	int rc = SLURM_SUCCESS;
-	char *addr, *dist = NULL;
+	char *dist = NULL;
 	char addrbuf[INET6_ADDRSTRLEN];
 
 	if (env == NULL)
@@ -843,14 +860,6 @@ int setup_env(env_t *env, bool preserve_env)
 		}
 	}
 
-	if (slurm_conf.slurmctld_addr)
-		addr = slurm_conf.slurmctld_addr;
-	else
-		addr = slurm_conf.control_addr[0];
-	setenvf(&env->env, "SLURM_WORKING_CLUSTER", "%s:%s:%d:%d:%d",
-	        slurm_conf.cluster_name, addr, slurm_conf.slurmctld_port,
-	        SLURM_PROTOCOL_VERSION, select_get_plugin_id());
-
 	return rc;
 }
 
@@ -952,7 +961,6 @@ extern char *uint32_compressed_to_str(uint32_t array_len,
  *	SLURM_JOB_NUM_NODES
  *	SLURM_JOB_NODELIST
  *	SLURM_JOB_CPUS_PER_NODE
- *	SLURM_NODE_ALIASES
  *	SLURM_NTASKS_PER_NODE
  *
  * dest OUT - array in which to the set environment variables
@@ -964,6 +972,7 @@ extern char *uint32_compressed_to_str(uint32_t array_len,
  *	SLURM_JOBID
  *	SLURM_NNODES
  *	SLURM_NODELIST
+ *	SLURM_NPROCS
  *	SLURM_TASKS_PER_NODE
  */
 extern int env_array_for_job(char ***dest,
@@ -1000,8 +1009,6 @@ extern int env_array_for_job(char ***dest,
 				    "%u", step_layout_req.num_hosts);
 	env_array_overwrite_het_fmt(dest, "SLURM_JOB_NODELIST", het_job_offset,
 				    "%s", alloc->node_list);
-	env_array_overwrite_het_fmt(dest, "SLURM_NODE_ALIASES", het_job_offset,
-				    "%s", alloc->alias_list);
 	env_array_overwrite_het_fmt(dest, "SLURM_JOB_PARTITION", het_job_offset,
 				    "%s", alloc->partition);
 
@@ -1206,7 +1213,6 @@ extern int env_array_for_job(char ***dest,
  *	SLURM_JOB_NUM_NODES
  *	SLURM_JOB_NODELIST
  *	SLURM_JOB_CPUS_PER_NODE
- *	SLURM_NODE_ALIASES
  *	ENVIRONMENT=BATCH
  *	HOSTNAME
  *
@@ -1246,8 +1252,9 @@ env_array_for_batch_job(char ***dest, const batch_job_launch_msg_t *batch,
 
 	/*
 	 * --ntasks-per-node no-longer sets num_tasks implicitly, so we need
-	 * need to calculate num_tasks here to make sure the environment
-	 * variable is correct.
+	 * need to calculate num_tasks here to make sure the
+	 * SLURM_TASKS_PER_NODE environment variable is correct. Also make sure
+	 * that the SLURM_NTASKS environment variable is set.
 	 *
 	 * --ntasks-per-tres still implicitly sets ntasks.
 	 * --ntasks-per-socket requires --ntasks in order to work.
@@ -1280,8 +1287,6 @@ env_array_for_batch_job(char ***dest, const batch_job_launch_msg_t *batch,
 	env_array_overwrite_fmt(dest, "SLURM_JOB_NODELIST", "%s", batch->nodes);
 	env_array_overwrite_fmt(dest, "SLURM_JOB_PARTITION", "%s",
 				batch->partition);
-	env_array_overwrite_fmt(dest, "SLURM_NODE_ALIASES", "%s",
-				batch->alias_list);
 
 	tmp = uint32_compressed_to_str(batch->num_cpu_groups,
 				       batch->cpus_per_node,
@@ -1319,8 +1324,10 @@ env_array_for_batch_job(char ***dest, const batch_job_launch_msg_t *batch,
 		/* keep around for old scripts */
 		env_array_overwrite_fmt(dest, "SLURM_NPROCS", "%u",
 					step_layout_req.num_tasks);
-	} else {
+	} else if (!step_layout_req.num_tasks) {
 		/*
+		 * Figure out num_tasks if it was not set by either
+		 * batch->ntasks or SLURM_NTASKS_PER_NODE above
 		 * Iterate over all kind of cluster nodes, and accum. the number
 		 * of tasks all the group can hold.
 		 */
@@ -1828,23 +1835,26 @@ void env_array_merge(char ***dest_array, const char **src_array)
 }
 
 /*
- * Merge the environment variables in src_array beginning with "SLURM" into the
- * array dest_array.  Any variables already found in dest_array will be
- * overwritten with the value from src_array.
+ * Merge the environment variables in src_array beginning with "SLURM" or
+ * SPANK_OPTION_ENV_PREFIX into the array dest_array. Any variables already
+ * found in dest_array will be overwritten with the value from src_array.
  */
-void env_array_merge_slurm(char ***dest_array, const char **src_array)
+void env_array_merge_slurm_spank(char ***dest_array, const char **src_array)
 {
 	char **ptr;
 	char name[256], *value;
+	int spank_len;
 
 	if (src_array == NULL)
 		return;
 
+	spank_len = strlen(SPANK_OPTION_ENV_PREFIX);
 	value = xmalloc(ENV_BUFSIZE);
 	for (ptr = (char **)src_array; *ptr != NULL; ptr++) {
 		if (_env_array_entry_splitter(*ptr, name, sizeof(name),
 					      value, ENV_BUFSIZE) &&
-		    (xstrncmp(name, "SLURM", 5) == 0))
+		    ((xstrncmp(name, "SLURM", 5) == 0) ||
+		     (xstrncmp(name, SPANK_OPTION_ENV_PREFIX, spank_len) == 0)))
 			env_array_overwrite(dest_array, name, value);
 	}
 	xfree(value);
@@ -1882,101 +1892,6 @@ static int _bracket_cnt(char *value)
 	return count;
 }
 
-/*
- * Load user environment from a specified file or file descriptor.
- *
- * This will read in a user specified file or fd, that is invoked
- * via the --export-file option in sbatch. The NAME=value entries must
- * be NULL separated to support special characters in the environment
- * definitions.
- *
- * (Note: This is being added to a minor release. For the
- * next major release, it might be a consideration to merge
- * this functionality with that of load_env_cache and update
- * env_cache_builder to use the NULL character.)
- */
-char **env_array_from_file(const char *fname)
-{
-	char *buf = NULL, *ptr = NULL, *eptr = NULL;
-	char *value, *p;
-	char **env = NULL;
-	char name[256];
-	int buf_size = BUFSIZ, buf_left;
-	int file_size = 0, tmp_size;
-	int separator = '\0';
-	int fd;
-
-	if (!fname)
-		return NULL;
-
-	/*
-	 * If file name is a numeric value, then it is assumed to be a
-	 * file descriptor.
-	 */
-	fd = (int)strtol(fname, &p, 10);
-	if ((*p != '\0') || (fd < 3) || (fd > sysconf(_SC_OPEN_MAX)) ||
-	    (fcntl(fd, F_GETFL) < 0)) {
-		fd = open(fname, O_RDONLY);
-		if (fd == -1) {
-			error("Could not open user environment file %s", fname);
-			return NULL;
-		}
-		verbose("Getting environment variables from %s", fname);
-	} else
-		verbose("Getting environment variables from fd %d", fd);
-
-	/*
-	 * Read in the user's environment data.
-	 */
-	buf = ptr = xmalloc(buf_size);
-	buf_left = buf_size;
-	while ((tmp_size = read(fd, ptr, buf_left))) {
-		if (tmp_size < 0) {
-			if (errno == EINTR)
-				continue;
-			error("read(environment_file): %m");
-			break;
-		}
-		buf_left  -= tmp_size;
-		file_size += tmp_size;
-		if (buf_left == 0) {
-			buf_size += BUFSIZ;
-			xrealloc(buf, buf_size);
-		}
-		ptr = buf + file_size;
-		buf_left = buf_size - file_size;
-	}
-	close(fd);
-
-	/*
-	 * Parse the buffer into individual environment variable names
-	 * and build the environment.
-	 */
-	env   = env_array_create();
-	value = xmalloc(ENV_BUFSIZE);
-	for (ptr = buf; ; ptr = eptr+1) {
-		eptr = strchr(ptr, separator);
-		if ((ptr == eptr) || (eptr == NULL))
-			break;
-		if (_env_array_entry_splitter(ptr, name, sizeof(name),
-					      value, ENV_BUFSIZE) &&
-		    (!_discard_env(name, value))) {
-			/*
-			 * Unset the SLURM_SUBMIT_DIR if it is defined so
-			 * that this new value does not get overwritten
-			 * in the subsequent call to env_array_merge().
-			 */
-			if (xstrcmp(name, "SLURM_SUBMIT_DIR") == 0)
-				unsetenv(name);
-			env_array_overwrite(&env, name, value);
-		}
-	}
-	xfree(buf);
-	xfree(value);
-
-	return env;
-}
-
 int env_array_to_file(const char *filename, const char **env_array,
 		      bool newline)
 {
@@ -1995,7 +1910,7 @@ int env_array_to_file(const char *filename, const char **env_array,
 		/* skip any env variables with a newline in newline mode */
 		if (newline && xstrstr(*p, "\n")) {
 			log_flag_hex(STEPS, *p, strlen(*p),
-				     "%s: skiping environment variable with newline",
+				     "%s: skipping environment variable with newline",
 				     __func__);
 			continue;
 		}
@@ -2023,7 +1938,7 @@ rwfail:
  */
 static char **_load_env_cache(const char *username)
 {
-	char fname[MAXPATHLEN];
+	char fname[PATH_MAX];
 	char *line, name[256], *value;
 	char **env = NULL;
 	FILE *fp;
@@ -2076,6 +1991,84 @@ static char **_load_env_cache(const char *username)
 	return env;
 }
 
+static int _child_fn(void *arg)
+{
+	char **tmp_env = NULL;
+	int devnull, fd = 3;
+	child_args_t *child_args = arg;
+	char *cmdstr;
+	const char *username;
+
+	username = child_args->username;
+	cmdstr = child_args->cmdstr;
+	tmp_env = child_args->tmp_env;
+
+	/*
+	 * Setting propagation and mounting our own /proc for this namespace.
+	 * This is done to ensure that this cloned process and its children
+	 * have coherent /proc contents with their virtual PIDs.
+	 * Check _clone_env_child to see namespace flags used in clone.
+	 */
+	if (child_args->setup_namespaces) {
+		if (mount("none", "/proc", NULL, MS_PRIVATE|MS_REC, NULL))
+			_exit(1);
+		if (mount("proc", "/proc", "proc",
+			  MS_NOSUID|MS_NOEXEC|MS_NODEV, NULL))
+			_exit(1);
+	}
+
+	if ((devnull = open("/dev/null", O_RDWR)) != -1) {
+		dup2(devnull, STDIN_FILENO);
+		dup2(devnull, STDERR_FILENO);
+	}
+	dup2(child_args->fildes[1], STDOUT_FILENO);
+
+	/* slow close all fds */
+	while (fd < child_args->rlimit)
+		close(fd++);
+
+	if (child_args->mode == 1)
+		execle(SUCMD, "su", username, "-c", cmdstr, NULL, tmp_env);
+	else if (child_args->mode == 2)
+		execle(SUCMD, "su", "-", username, "-c", cmdstr, NULL, tmp_env);
+	else {	/* Default system configuration */
+#ifdef LOAD_ENV_NO_LOGIN
+		execle(SUCMD, "su", username, "-c", cmdstr, NULL, tmp_env);
+#else
+		execle(SUCMD, "su", "-", username, "-c", cmdstr, NULL, tmp_env);
+#endif
+	}
+	if (devnull >= 0)	/* Avoid Coverity resource leak notification */
+		(void) close(devnull);
+
+	_exit(1);
+}
+
+#if !defined(__APPLE__) && !defined(__FreeBSD__) && !defined(__NetBSD__)
+static int _clone_env_child(child_args_t *child_args)
+{
+	char *child_stack;
+	child_stack = mmap(NULL, STACK_SIZE, PROT_READ | PROT_WRITE,
+			   MAP_PRIVATE | MAP_ANONYMOUS | MAP_STACK, -1, 0);
+	if (child_stack == MAP_FAILED) {
+		error("Cannot allocate stack for child: %m");
+		return -1;
+	}
+	/*
+	 * In Linux (since 2.6.24), use CLONE_NEWPID to clone the child into a
+	 * new pid namespace. We are not into a job cgroup so we want to be
+	 * able to terminate any possible background process, specially because
+	 * we're using sudo here and running some user scripts (e.g. .bashrc).
+	 *
+	 * Killing the 'child' pid will kill all the namespace, since in the
+	 * namespace, this 'child' is pid 1.
+	 */
+	child_args->setup_namespaces = true;
+	return clone(_child_fn, child_stack + STACK_SIZE,
+		     (SIGCHLD|CLONE_NEWPID|CLONE_NEWNS), child_args);
+}
+#endif
+
 /*
  * Return an array of strings representing the specified user's default
  * environment variables following a two-prongged approach.
@@ -2096,7 +2089,7 @@ static char **_load_env_cache(const char *username)
 char **env_array_user_default(const char *username, int timeout, int mode,
 			      bool no_cache)
 {
-	char *line = NULL, *last = NULL, name[MAXPATHLEN], *value, *buffer;
+	char *line = NULL, *last = NULL, name[PATH_MAX], *value, *buffer;
 	char **env = NULL;
 	char *starttoken = "XXXXSLURMSTARTPARSINGHEREXXXX";
 	char *stoptoken  = "XXXXSLURMSTOPPARSINGHEREXXXXX";
@@ -2105,9 +2098,11 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 	int fildes[2], found, fval, len, rc, timeleft;
 	int buf_read, buf_rem, config_timeout;
 	pid_t child;
+	child_args_t child_args = {0};
 	struct timeval begin, now;
 	struct pollfd ufds;
 	struct stat buf;
+	struct rlimit rlim;
 
 	if (geteuid() != (uid_t)0) {
 		error("SlurmdUser must be root to use --get-user-env");
@@ -2142,41 +2137,33 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 		return NULL;
 	}
 
+	child_args.mode = mode;
+	child_args.fildes = fildes;
+	child_args.username = username;
+	child_args.cmdstr = cmdstr;
+	child_args.setup_namespaces = false;
+	child_args.tmp_env = env_array_create();
+	env_array_overwrite(&child_args.tmp_env, "ENVIRONMENT", "BATCH");
+	if (getrlimit(RLIMIT_NOFILE, &rlim) < 0) {
+		error("getrlimit(RLIMIT_NOFILE): %m");
+		rlim.rlim_cur = 4096;
+	}
+	child_args.rlimit = rlim.rlim_cur;
+
+#if defined(__APPLE__) || defined(__FreeBSD__) || defined(__NetBSD__)
 	child = fork();
 	if (child == -1) {
 		fatal("fork: %m");
 		return NULL;
 	}
-	if (child == 0) {
-		char **tmp_env = NULL;
-		int devnull;
-		tmp_env = env_array_create();
-		env_array_overwrite(&tmp_env, "ENVIRONMENT", "BATCH");
-		setpgid(0, 0);
-
-		if ((devnull = open("/dev/null", O_RDONLY)) == -1)
-			error("%s: open(/dev/null): %m", __func__);
-		dup2(devnull, STDIN_FILENO);
-		dup2(fildes[1], STDOUT_FILENO);
-		dup2(devnull, STDERR_FILENO);
-		closeall(3);
-
-		if      (mode == 1)
-			execle(SUCMD, "su", username, "-c", cmdstr, NULL, tmp_env);
-		else if (mode == 2)
-			execle(SUCMD, "su", "-", username, "-c", cmdstr, NULL, tmp_env);
-		else {	/* Default system configuration */
-#ifdef LOAD_ENV_NO_LOGIN
-			execle(SUCMD, "su", username, "-c", cmdstr, NULL, tmp_env);
+	if (child == 0)
+		_child_fn(&child_args);
 #else
-			execle(SUCMD, "su", "-", username, "-c", cmdstr, NULL, tmp_env);
-#endif
-		}
-		if (devnull >= 0)	/* Avoid Coverity resource leak notification */
-			(void) close(devnull);
-		_exit(1);
+	if ((child = _clone_env_child(&child_args)) == -1) {
+		fatal("clone: %m");
+		return NULL;
 	}
-
+#endif
 	close(fildes[1]);
 	if ((fval = fcntl(fildes[0], F_GETFL, 0)) < 0)
 		error("fcntl(F_GETFL) failed: %m");
@@ -2240,6 +2227,8 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 		}
 	}
 	close(fildes[0]);
+	env_array_free(child_args.tmp_env);
+
 	for (config_timeout=0; ; config_timeout++) {
 		kill(-child, SIGKILL);	/* Typically a no-op */
 		if (config_timeout)
@@ -2323,6 +2312,16 @@ char **env_array_user_default(const char *username, int timeout, int mode,
 	return env;
 }
 
+static void _set_ext_launcher_hydra(char ***dest, char *b_env, char *extra)
+{
+	char *bootstrap = getenv(b_env);
+
+	if (!bootstrap || !xstrcmp(bootstrap, "slurm")) {
+		env_array_append(dest, b_env, "slurm");
+		env_array_append(dest, extra, "--external-launcher");
+	}
+}
+
 /*
  * Set TRES related env vars. Set here rather than env_array_for_job() since
  * we don't have array of opt values and the raw values are not stored in the
@@ -2345,11 +2344,6 @@ extern void set_env_from_opts(slurm_opt_t *opt, char ***dest,
 					    het_job_offset, "%s",
 					    opt->gpus);
 	}
-	if (opt->gpu_bind) {
-		env_array_overwrite_het_fmt(dest, "SLURM_GPU_BIND",
-					    het_job_offset, "%s",
-					    opt->gpu_bind);
-	}
 	if (opt->gpu_freq) {
 		env_array_overwrite_het_fmt(dest, "SLURM_GPU_FREQ",
 					    het_job_offset, "%s",
@@ -2365,11 +2359,6 @@ extern void set_env_from_opts(slurm_opt_t *opt, char ***dest,
 					    het_job_offset, "%s",
 					    opt->gpus_per_socket);
 	}
-	if (opt->gpus_per_task) {
-		env_array_overwrite_het_fmt(dest, "SLURM_GPUS_PER_TASK",
-					    het_job_offset, "%s",
-					    opt->gpus_per_task);
-	}
 	if (opt->mem_per_gpu != NO_VAL64) {
 		env_array_overwrite_het_fmt(dest, "SLURM_MEM_PER_GPU",
 					    het_job_offset, "%"PRIu64,
@@ -2380,6 +2369,30 @@ extern void set_env_from_opts(slurm_opt_t *opt, char ***dest,
 					    het_job_offset, "%s",
 					    opt->tres_per_task);
 	}
+	if (opt->tres_bind) {
+		env_array_overwrite_het_fmt(dest, "SLURM_TRES_BIND",
+					    het_job_offset, "%s",
+					    opt->tres_bind);
+	}
+
+	/*
+	 * In the case that an external launcher (mpirun) is launching instead
+	 * of srun let the srun it launches to treat the request differently.
+	 */
+	env_array_append(dest, "OMPI_MCA_plm_slurm_args",
+			 "--external-launcher");
+	env_array_append(dest, "PRTE_MCA_plm_slurm_args",
+			 "--external-launcher");
+
+	/*
+	 * Some mpirun implementations like intel will pass the
+	 * bootstrap exec extra args to any bootstrap method (e.g. ssh,
+	 * rsh), so force 'slurm' bootstrap if no other one is set.
+	 */
+	_set_ext_launcher_hydra(dest, "HYDRA_BOOTSTRAP",
+				"HYDRA_LAUNCHER_EXTRA_ARGS");
+	_set_ext_launcher_hydra(dest, "I_MPI_HYDRA_BOOTSTRAP",
+				"I_MPI_HYDRA_BOOTSTRAP_EXEC_EXTRA_ARGS");
 }
 
 extern char *find_quote_token(char *tmp, char *sep, char **last)
@@ -2489,3 +2502,24 @@ extern char **env_array_exclude(const char **env, const regex_t *regex)
 
 	return purged;
 }
+
+extern void set_prio_process_env(void)
+{
+        int retval;
+
+        errno = 0; /* needed to detect a real failure since prio can be -1 */
+
+        if ((retval = getpriority(PRIO_PROCESS, 0)) == -1)  {
+                if (errno) {
+                        error("getpriority(PRIO_PROCESS): %m");
+                        return;
+                }
+        }
+
+        if (setenvf(NULL, "SLURM_PRIO_PROCESS", "%d", retval) < 0) {
+                error("unable to set SLURM_PRIO_PROCESS in environment");
+                return;
+        }
+
+        debug("propagating SLURM_PRIO_PROCESS=%d", retval);
+}
