@@ -7,7 +7,7 @@
  *  Portions Copyright (C) 2008 Vijay Ramasubramanian.
  *  Portions Copyright (C) 2010-2016 SchedMD <https://www.schedmd.com>.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
- *  Written by Morris Mette <jette1@llnl.gov>.
+ *  Written by Morris Jette <jette1@llnl.gov>.
  *  CODE-OCEC-09-009. All rights reserved.
  *
  *  This file is part of Slurm, a resource management program.
@@ -50,94 +50,88 @@
 #include "src/common/parse_config.h"
 #include "src/common/run_in_daemon.h"
 
-extern slurm_ctl_conf_t slurmctld_conf;
+extern slurm_conf_t slurm_conf;
 extern char *default_slurm_config_file;
 extern char *default_plugin_path;
-extern char *default_plugstack;
 
 #ifndef NDEBUG
 extern uint16_t drop_priv_flag;
 #endif
 
-#define ACCOUNTING_ENFORCE_ASSOCS 0x0001
-#define ACCOUNTING_ENFORCE_LIMITS 0x0002
-#define ACCOUNTING_ENFORCE_WCKEYS 0x0004
-#define ACCOUNTING_ENFORCE_QOS    0x0008
-#define ACCOUNTING_ENFORCE_SAFE   0x0010
-#define ACCOUNTING_ENFORCE_NO_JOBS 0x0020
-#define ACCOUNTING_ENFORCE_NO_STEPS 0x0040
-#define ACCOUNTING_ENFORCE_TRES   0x0080
+/*
+ * We can't include node_conf.h to get node_record_t because node_conf.h
+ * includes read_config.h and creates a circular dependency. We create the
+ * typedef so that we don't have to move the struct around.
+ */
+#ifndef node_record_t
+typedef struct node_record node_record_t;
+#endif
+
+#define ACCOUNTING_ENFORCE_ASSOCS SLURM_BIT(0)
+#define ACCOUNTING_ENFORCE_LIMITS SLURM_BIT(1)
+#define ACCOUNTING_ENFORCE_WCKEYS SLURM_BIT(2)
+#define ACCOUNTING_ENFORCE_QOS    SLURM_BIT(3)
+#define ACCOUNTING_ENFORCE_SAFE   SLURM_BIT(4)
+#define ACCOUNTING_ENFORCE_NO_JOBS SLURM_BIT(5)
+#define ACCOUNTING_ENFORCE_NO_STEPS SLURM_BIT(6)
+#define ACCOUNTING_ENFORCE_TRES   SLURM_BIT(7)
 
 #define DEFAULT_ACCOUNTING_TRES  "cpu,mem,energy,node,billing,fs/disk,vmem,pages"
 #define DEFAULT_ACCOUNTING_DB      "slurm_acct_db"
 #define DEFAULT_ACCOUNTING_ENFORCE  0
-#define DEFAULT_ACCOUNTING_STORAGE_TYPE "accounting_storage/none"
 #define DEFAULT_AUTH_TYPE          "auth/munge"
 #define DEFAULT_AUTH_TOKEN_LIFESPAN 1800
 #define DEFAULT_BATCH_START_TIMEOUT 10
+#define DEFAULT_BCAST_EXCLUDE       "/lib,/usr/lib,/lib64,/usr/lib64"
 #define DEFAULT_COMPLETE_WAIT       0
 #define DEFAULT_CRED_TYPE           "cred/munge"
 #define DEFAULT_EPILOG_MSG_TIME     2000
-#define DEFAULT_EXT_SENSORS_TYPE    "ext_sensors/none"
 #define DEFAULT_FIRST_JOB_ID        1
 #define DEFAULT_GET_ENV_TIMEOUT     2
+#define DEFAULT_GETNAMEINFO_CACHE_TIMEOUT 60
 #define DEFAULT_GROUP_TIME          600
 #define DEFAULT_GROUP_FORCE         1	/* if set, update group membership
 					 * info even if no updates to
 					 * /etc/group file */
 /* NOTE: DEFAULT_INACTIVE_LIMIT must be 0 for Blue Gene/L systems */
 #define DEFAULT_INACTIVE_LIMIT      0
-#define DEFAULT_JOB_ACCT_GATHER_TYPE  "jobacct_gather/none"
-#define JOB_ACCT_GATHER_TYPE_NONE "jobacct_gather/none"
+#define DEFAULT_INTERACTIVE_STEP_OPTS "--interactive --preserve-env --pty $SHELL"
 #define DEFAULT_JOB_ACCT_GATHER_FREQ  "30"
-#define DEFAULT_ACCT_GATHER_ENERGY_TYPE "acct_gather_energy/none"
-#define DEFAULT_ACCT_GATHER_PROFILE_TYPE "acct_gather_profile/none"
-#define DEFAULT_ACCT_GATHER_INTERCONNECT_TYPE "acct_gather_interconnect/none"
-#define DEFAULT_ACCT_GATHER_FILESYSTEM_TYPE "acct_gather_filesystem/none"
-#define ACCOUNTING_STORAGE_TYPE_NONE "accounting_storage/none"
-#define DEFAULT_CORE_SPEC_PLUGIN    "core_spec/none"
 #define DEFAULT_ENFORCE_PART_LIMITS 0
-#define DEFAULT_JOB_COMP_TYPE       "jobcomp/none"
-#define DEFAULT_JOB_COMP_LOC        "/var/log/slurm_jobcomp.log"
-#define DEFAULT_JOB_COMP_DB         "slurm_jobcomp_db"
 #if defined HAVE_NATIVE_CRAY
 #  define DEFAULT_ALLOW_SPEC_RESOURCE_USAGE 1
-#  define DEFAULT_JOB_CONTAINER_PLUGIN  "job_container/cncu"
 #else
 #  define DEFAULT_ALLOW_SPEC_RESOURCE_USAGE 0
-#  define DEFAULT_JOB_CONTAINER_PLUGIN "job_container/none"
 #endif
-#define DEFAULT_KEEP_ALIVE_TIME     (NO_VAL16)
+#define DEFAULT_KEEPALIVE_TIME (NO_VAL)
+#define DEFAULT_KEEPALIVE_INTERVAL (NO_VAL)
+#define DEFAULT_KEEPALIVE_PROBES (NO_VAL)
 #define DEFAULT_KILL_ON_BAD_EXIT    0
 #define DEFAULT_KILL_TREE           0
 #define DEFAULT_KILL_WAIT           30
-#define DEFAULT_LAUNCH_TYPE         "launch/slurm"
 #define DEFAULT_MAIL_PROG           "/bin/mail"
 #define DEFAULT_MAIL_PROG_ALT       "/usr/bin/mail"
 #define DEFAULT_MAX_ARRAY_SIZE      1001
+#define DEFAULT_MAX_BATCH_REQUEUE   5
 #define DEFAULT_MAX_DBD_MSGS        10000
 #define DEFAULT_MAX_JOB_COUNT       10000
 #define DEFAULT_MAX_JOB_ID          0x03ff0000
 #define DEFAULT_MAX_STEP_COUNT      40000
-#define DEFAULT_MCS_PLUGIN          "mcs/none"
 #define DEFAULT_MEM_PER_CPU         0
 #define DEFAULT_MAX_MEM_PER_CPU     0
 #define DEFAULT_MIN_JOB_AGE         300
-#define DEFAULT_MPI_DEFAULT         "none"
 #define DEFAULT_MSG_AGGR_WINDOW_MSGS 1
 #define DEFAULT_MSG_AGGR_WINDOW_TIME 100
 #define DEFAULT_MSG_TIMEOUT         10
-#define DEFAULT_POWER_PLUGIN        ""
 #if defined WITH_CGROUP
 #  define DEFAULT_PROCTRACK_TYPE      "proctrack/cgroup"
 #else
 #  define DEFAULT_PROCTRACK_TYPE      "proctrack/pgid"
 #endif
-#define DEFAULT_PREEMPT_TYPE        "preempt/none"
 #define DEFAULT_PREP_PLUGINS        "prep/script"
 #define DEFAULT_PRIORITY_DECAY      604800 /* 7 days */
 #define DEFAULT_PRIORITY_CALC_PERIOD 300 /* in seconds */
-#define DEFAULT_PRIORITY_TYPE       "priority/basic"
+#define DEFAULT_PRIORITY_TYPE       "priority/multifactor"
 #define DEFAULT_RECONF_KEEP_PART_STATE 0
 #define DEFAULT_RETURN_TO_SERVICE   0
 #define DEFAULT_RESUME_RATE         300
@@ -150,9 +144,8 @@ extern uint16_t drop_priv_flag;
 #if defined HAVE_NATIVE_CRAY
 #  define DEFAULT_SELECT_TYPE       "select/cray_aries"
 #else
-#  define DEFAULT_SELECT_TYPE       "select/linear"
+#  define DEFAULT_SELECT_TYPE       "select/cons_tres"
 #endif
-#define DEFAULT_SITE_FACTOR_PLUGIN  "site_factor/none"
 #define DEFAULT_SLURMCTLD_PIDFILE   "/var/run/slurmctld.pid"
 #define DEFAULT_SLURMCTLD_TIMEOUT   120
 #define DEFAULT_SLURMD_PIDFILE      "/var/run/slurmd.pid"
@@ -166,12 +159,6 @@ extern uint16_t drop_priv_flag;
 #define DEFAULT_SUSPEND_RATE        60
 #define DEFAULT_SUSPEND_TIME        0
 #define DEFAULT_SUSPEND_TIMEOUT     30
-#if defined HAVE_NATIVE_CRAY
-#  define DEFAULT_SWITCH_TYPE         "switch/cray_aries"
-#else
-#  define DEFAULT_SWITCH_TYPE         "switch/none"
-#endif
-#define DEFAULT_TASK_PLUGIN         "task/none"
 #define DEFAULT_TCP_TIMEOUT         2
 #define DEFAULT_TMP_FS              "/tmp"
 #if defined HAVE_3D
@@ -180,8 +167,12 @@ extern uint16_t drop_priv_flag;
 #  define DEFAULT_TOPOLOGY_PLUGIN     "topology/none"
 #endif
 #define DEFAULT_WAIT_TIME           0
-#  define DEFAULT_TREE_WIDTH        50
+#define DEFAULT_TREE_WIDTH	    16
 #define DEFAULT_UNKILLABLE_TIMEOUT  60 /* seconds */
+#define DEFAULT_BATCH_SCRIPT_LIMIT (4 * 1024 * 1024) /* 4MB */
+#define MAX_BATCH_SCRIPT_SIZE (512 * 1024 * 1024) /* 512MB */
+#define DEFAULT_MAX_SUBMIT_LINE_SIZE (1024 * 1024) /* 1MB */
+#define MAX_MAX_SUBMIT_LINE_SIZE (2 * 1024 * 1024) /* 2MB */
 
 /* MAX_TASKS_PER_NODE is defined in slurm.h
  */
@@ -213,7 +204,7 @@ typedef struct slurm_conf_node {
 	uint16_t cpus;		/* count of cpus running on the node */
 	char *cpu_spec_list;	/* arbitrary list of specialized cpus */
 	uint16_t boards; 	/* number of boards per node */
-	uint16_t sockets;       /* number of sockets per node */
+	uint16_t tot_sockets;   /* number of sockets per node */
 	uint16_t cores;         /* number of cores per CPU */
 	uint16_t core_spec_cnt;	/* number of specialized cores */
 	uint16_t threads;       /* number of threads per core */
@@ -249,15 +240,15 @@ typedef struct slurm_conf_partition {
 				 * NULL indicates all */
 	char *deny_qos;		/* comma delimited list of denied qos,
 				 * NULL indicates all */
-	uint16_t disable_root_jobs; /* if set then user root can't run
-				     * jobs if NO_VAL use global
-				     * default */
-	uint16_t exclusive_user; /* 1 if node allocations by user */
+	uint8_t disable_root_jobs; /* if set then user root can't run jobs
+				    * if NO_VAL8, use global default */
+	bool exclusive_user; /* true if node allocations by user */
 	uint32_t grace_time;	/* default grace time for partition */
 	bool     hidden_flag;	/* 1 if hidden by default */
 	List job_defaults_list;	/* List of job_defaults_t elements */
 	bool     lln_flag;	/* 1 if nodes are selected in LLN order */
 	uint32_t max_cpus_per_node; /* maximum allocated CPUs per node */
+	uint32_t max_cpus_per_socket; /* maximum allocated CPUs per socket */
 	uint16_t max_share;	/* number of jobs to gang schedule */
 	uint32_t max_time;	/* minutes or INFINITE */
 	uint64_t max_mem_per_cpu; /* maximum MB memory per allocated CPU */
@@ -267,15 +258,23 @@ typedef struct slurm_conf_partition {
 	char 	*nodes;		/* comma delimited list names of nodes */
 	uint16_t over_time_limit; /* job's time limit can be exceeded by this
 				   * number of minutes before cancellation */
+	bool power_down_on_idle; /* true if nodes POWER_DOWN upon returning to
+				  * IDLE */
 	uint16_t preempt_mode;	/* See PREEMPT_MODE_* in slurm/slurm.h */
 	uint16_t priority_job_factor;	/* job priority weight factor */
 	uint16_t priority_tier;	/* tier for scheduling and preemption */
 	char    *qos_char;      /* Name of QOS associated with partition */
 	bool     req_resv_flag; /* 1 if partition can only be used in a
 				 * reservation */
+	uint16_t resume_timeout; /* time required in order to perform a node
+				  * resume operation */
 	bool     root_only_flag;/* 1 if allocate/submit RPC can only be
 				   issued by user root */
 	uint16_t state_up;	/* for states see PARTITION_* in slurm.h */
+	uint32_t suspend_time;  /* node idle for this long before power save
+				 * mode */
+	uint16_t suspend_timeout; /* time required in order to perform a node
+				   * suspend operation */
 	uint32_t total_nodes;	/* total number of nodes in the partition */
 	uint32_t total_cpus;	/* total number of cpus in the partition */
 } slurm_conf_partition_t;
@@ -337,19 +336,49 @@ extern int job_defaults_list(char *in_str, List *out_list);
 extern char *job_defaults_str(List in_list);
 
 /* Pack a job_defaults_t element. Used by slurm_pack_list() */
-extern void job_defaults_pack(void *in, uint16_t protocol_version, Buf buffer);
+extern void job_defaults_pack(void *in, uint16_t protocol_version,
+			      buf_t *buffer);
 
 /* Unpack a job_defaults_t element. Used by slurm_pack_list() */
 extern int job_defaults_unpack(void **out, uint16_t protocol_version,
-			       Buf buffer);
+			       buf_t *buffer);
 
 /*
- * list_find_frontend - find an entry in the front_end list, see list.h for
- *	documentation
- * IN key - is feature name or NULL for all features
- * RET 1 if found, 0 otherwise
+ * slurm_reset_alias() for each node in alias_list
+ *
+ * IN alias_list - string with sets of node name, communication address in []
+ * 	and hostname. Each element in the set if colon separated and
+ * 	each set is comma separated.
+ * 	eg.: ec0:[1.2.3.4]:foo,ec1:[1.2.3.5]:bar
+ * RET return SLURM_SUCCESS on success, SLURM_ERROR otherwise.
  */
-extern int list_find_frontend (void *front_end_entry, void *key);
+extern int set_nodes_alias(const char *alias_list);
+
+/* Send conf_hashtbl to the stepd */
+extern int read_conf_send_stepd(int fd);
+
+/* Receive conf_hashtbl from the slurmd */
+extern void read_conf_recv_stepd(int fd);
+
+/*
+ * Allocate memory for a config_key_pair_t pointer and initialize it by
+ * duplicating the key-value arguments. Append the resulting pair to the
+ * argument list if it's not NULL.
+ *
+ * IN: list_t *key_pair_list
+ * IN: char *key
+ * IN: char *value
+ */
+extern void read_config_add_key_pair(list_t *key_pair_list,
+				     char *key,
+				     char *value);
+
+
+/*
+ * slurm_conf_init_stepd - Since the stepd does not read in the file and
+ * receives it from the slurm we need to call a different function to do this.
+ */
+extern void slurm_conf_init_stepd(void);
 
 /*
  * slurm_conf_init - load the slurm configuration from the a file.
@@ -394,11 +423,12 @@ void slurm_conf_install_fork_handlers(void);
  */
 extern int slurm_conf_destroy(void);
 
-extern slurm_ctl_conf_t *slurm_conf_lock(void);
+extern slurm_conf_t *slurm_conf_lock(void);
 
 extern void slurm_conf_unlock(void);
 
 
+extern int slurm_conf_check_addr(const char *node_name, bool *dynamic);
 /*
  * Set "ptr_array" with the pointer to an array of pointers to
  * slurm_conf_frontend_t structures.
@@ -445,6 +475,16 @@ extern int slurm_conf_nodeset_array(slurm_conf_nodeset_t **ptr_array[]);
 extern void slurm_reset_alias(char *node_name, char *node_addr,
 			      char *node_hostname);
 
+/*
+ * Return NodeAddr (if set) for a given NodeName, or NULL
+ *
+ * Returned string was allocated with xmalloc(), and must be freed by
+ * the caller using xfree().
+ *
+ * NOTE: Caller must NOT be holding slurm_conf_lock().
+ */
+extern char* slurm_conf_get_address(const char *node_name);
+
 /*
  * slurm_conf_get_hostname - Return the NodeHostname for given NodeName
  *
@@ -465,7 +505,7 @@ extern char *slurm_conf_get_nodename(const char *node_hostname);
 
 /*
  * slurm_conf_get_aliases - Return all the nodes NodeName value
- * associated to a given NodeHostname (usefull in case of multiple-slurmd
+ * associated to a given NodeHostname (useful in case of multiple-slurmd
  * to get the list of virtual nodes associated with a real node)
  *
  * NOTE: Call xfree() to release returned value's memory.
@@ -481,14 +521,6 @@ extern char *slurm_conf_get_aliases(const char *node_hostname);
  */
 extern char *slurm_conf_get_nodeaddr(const char *node_hostname);
 
-/*
- * slurm_conf_get_nodename_from_addr - Return the NodeName for given NodeAddr
- *
- * NOTE: Call xfree() to release returned value's memory.
- * NOTE: Caller must NOT be holding slurm_conf_lock().
- */
-extern char *slurm_conf_get_nodename_from_addr(const char *node_addr);
-
 /*
  * slurm_conf_get_aliased_nodename - Return the NodeName matching an alias
  * of the local hostname
@@ -510,13 +542,6 @@ extern char *slurm_conf_get_aliased_nodename(void);
  */
 extern char *slurm_conf_get_bcast_address(const char *node_name);
 
-/*
- * slurm_conf_get_port - Return the port for a given NodeName
- *
- * NOTE: Caller must NOT be holding slurm_conf_lock().
- */
-extern uint16_t slurm_conf_get_port(const char *node_name);
-
 /*
  * slurm_conf_get_addr - Return the slurm_addr_t for a given NodeName in
  *	the parameter "address".  The return code is SLURM_SUCCESS on success,
@@ -528,46 +553,32 @@ extern int slurm_conf_get_addr(const char *node_name, slurm_addr_t *address,
 			       uint16_t flags);
 
 /*
- * slurm_conf_get_cpus_bsct -
- * Return the cpus, boards, sockets, cores, and threads configured for a
- * given NodeName
- * Returns SLURM_SUCCESS on success, SLURM_ERROR on failure.
+ * Parse slurm.conf NodeName line and return single slurm_conf_node_t*.
  *
- * NOTE: Caller must NOT be holding slurm_conf_lock().
+ * IN nodeline - NodeName= line string.
+ * OUT out_hashtbl - ptr to the generated hashtable so it can be deleted by
+ *                   caller after using the slurm_conf_node_t*. Currently, not a
+ *                   way to disassociate items from the hashtbl.
+ * RET slurm_conf_t* on success, NULL otherwise.
  */
-extern int slurm_conf_get_cpus_bsct(const char *node_name,
-				    uint16_t *cpus, uint16_t *boards,
-				    uint16_t *sockets, uint16_t *cores,
-				    uint16_t *threads);
-
-/*
- * slurm_conf_get_res_spec_info - Return resource specialization info
- * for a given NodeName
- * Returns SLURM_SUCCESS on success, SLURM_ERROR on failure.
- *
- * NOTE: Caller must NOT be holding slurm_conf_lock().
- */
-extern int slurm_conf_get_res_spec_info(const char *node_name,
-					char **cpu_spec_list,
-					uint16_t *core_spec_cnt,
-					uint64_t *mem_spec_limit);
+extern slurm_conf_node_t *slurm_conf_parse_nodeline(const char *nodeline,
+						    s_p_hashtbl_t **out_hashtbl);
 
 /*
  * init_slurm_conf - initialize or re-initialize the slurm configuration
- *	values defaults (NULL or NO_VAL). Note that the configuration
+ *	values to defaults (NULL or NO_VAL). Note that the configuration
  *	file pathname (slurm_conf) is not changed.
  * IN/OUT ctl_conf_ptr - pointer to data structure to be initialized
  */
-extern void init_slurm_conf (slurm_ctl_conf_t *ctl_conf_ptr);
+extern void init_slurm_conf(slurm_conf_t *ctl_conf_ptr);
 
 /*
- * free_slurm_conf - free all storage associated with a slurm_ctl_conf_t.
+ * free_slurm_conf - free all storage associated with a slurm_conf_t.
  * IN/OUT ctl_conf_ptr - pointer to data structure to be freed
  * IN purge_node_hash - purge system-wide node hash table if set,
  *			set to zero if clearing private copy of config data
  */
-extern void free_slurm_conf (slurm_ctl_conf_t *ctl_conf_ptr,
-			     bool purge_node_hash);
+extern void free_slurm_conf(slurm_conf_t *ctl_conf_ptr, bool purge_node_hash);
 
 /*
  * gethostname_short - equivalent to gethostname(), but return only the first
@@ -587,7 +598,8 @@ extern int gethostname_short(char *name, size_t len);
  * Returns an xmalloc()ed string which the caller must free with xfree().
  */
 extern char *slurm_conf_expand_slurmd_path(const char *path,
-					   const char *node_name);
+					   const char *node_name,
+					   const char *host_name);
 
 /*
  * prolog_flags2str - convert a PrologFlags uint16_t to the equivalent string
@@ -611,7 +623,7 @@ extern char *debug_flags2str(uint64_t debug_flags);
  * debug_str2flags - Convert a DebugFlags string to the equivalent uint64_t
  * Returns SLURM_ERROR if invalid
  */
-extern int debug_str2flags(char *debug_flags, uint64_t *flags_out);
+extern int debug_str2flags(const char *debug_flags, uint64_t *flags_out);
 
 /*
  * reconfig_flags2str - convert a ReconfigFlags uint16_t to the equivalent string
@@ -627,24 +639,24 @@ extern uint16_t reconfig_str2flags(char *reconfig_flags);
 
 extern void destroy_config_plugin_params(void *object);
 extern void pack_config_plugin_params(void *in, uint16_t protocol_version,
-				      Buf buff);
+				      buf_t *buff);
 extern int unpack_config_plugin_params(void **object, uint16_t protocol_version,
-				       Buf buff);
+				       buf_t *buff);
 extern void pack_config_plugin_params_list(void *in, uint16_t protocol_version,
-					   Buf buff);
+					   buf_t *buff);
 extern int unpack_config_plugin_params_list(void **object,
 					    uint16_t protocol_version,
-					    Buf buff);
+					    buf_t *buff);
 
 extern void destroy_config_key_pair(void *object);
 extern void pack_key_pair_list(void *key_pairs, uint16_t protocol_version,
-			       Buf buffer);
+			       buf_t *buffer);
 extern int unpack_key_pair_list(void **key_pairs, uint16_t protocol_version,
-				Buf buffer);
+				buf_t *buffer);
 extern void pack_config_key_pair(void *in, uint16_t protocol_version,
-				 Buf buffer);
+				 buf_t *buffer);
 extern int unpack_config_key_pair(void **object, uint16_t protocol_version,
-				  Buf buffer);
+				  buf_t *buffer);
 
 extern int sort_key_pairs(void *v1, void *v2);
 /*
@@ -669,4 +681,32 @@ extern char *xlate_features(char *job_features);
 extern int add_remote_nodes_to_conf_tbls(char *node_list,
 					 slurm_addr_t *node_addrs);
 
+/*
+ * Add record to conf hash tables from node_record_t.
+ */
+extern void slurm_conf_add_node(node_record_t *node_ptr);
+
+/*
+ * Remove node from node conf hash tables.
+ */
+extern void slurm_conf_remove_node(char *node_name);
+
+#ifdef HAVE_FRONT_END
+/*
+ * Return the frontend port for the given hostname.
+ */
+extern uint16_t slurm_conf_get_frontend_port(char *node_hostname);
+#endif
+
+/*
+ * Get substring from a csv-style string.
+ *
+ * IN opts - csv-style string to be parsed
+ * IN arg - the search string + delimiter between option and its value
+ * 	(EG: "planet=")
+ *
+ * RET xmalloc()'d string after the delimiter, and before next ","
+ */
+extern char *conf_get_opt_str(const char *opts, const char *arg);
+
 #endif /* !_READ_CONFIG_H */
