@@ -103,7 +103,7 @@ enum {
 	LONG_OPT_EXACT,
 	LONG_OPT_EXCLUSIVE,
 	LONG_OPT_EXPORT,
-	LONG_OPT_EXPORT_FILE,
+	LONG_OPT_EXTERNAL_LAUNCHER,
 	LONG_OPT_EXTRA,
 	LONG_OPT_GET_USER_ENV,
 	LONG_OPT_GID,
@@ -180,6 +180,8 @@ enum {
 	LONG_OPT_THREADSPERCORE,
 	LONG_OPT_TIME_MIN,
 	LONG_OPT_TMP,
+	LONG_OPT_TREE_WIDTH,
+	LONG_OPT_TRES_BIND,
 	LONG_OPT_TRES_PER_JOB,
 	LONG_OPT_TRES_PER_TASK,
 	LONG_OPT_UID,
@@ -210,7 +212,6 @@ typedef struct {
 typedef struct {
 	char *array_inx;		/* --array			*/
 	char *batch_features;		/* --batch			*/
-	char *export_file;		/* --export-file=file		*/
 	bool ignore_pbs;		/* --ignore-pbs			*/
 	int minsockets;			/* --minsockets=n		*/
 	int mincores;			/* --mincores=n			*/
@@ -245,6 +246,7 @@ typedef struct {
 	cpu_bind_type_t cpu_bind_type;	/* --cpu-bind			*/
 	bool debugger_test;		/* --debugger-test		*/
 	bool disable_status;		/* --disable-status		*/
+	bool external_launcher;		/* --external-launcher		*/
 	char *epilog;			/* --epilog			*/
 	bool exact;			/* --exact			*/
 	bool exclusive;			/* --exclusive			*/
@@ -278,6 +280,7 @@ typedef struct {
 	char *task_prolog;		/* --task-prolog		*/
 	bool test_exec;			/* test_exec set		*/
 	bool test_only;			/* --test-only			*/
+	uint16_t tree_width;		/* --treewidth			*/
 	bool unbuffered;		/* --unbuffered			*/
 	bool whole;			/* --whole			*/
 } srun_opt_t;
@@ -318,7 +321,7 @@ typedef struct {
 	bool nodes_set;			/* nodes explicitly set		*/
 	int sockets_per_node;		/* --sockets-per-node=n		*/
 	int cores_per_socket;		/* --cores-per-socket=n		*/
-	uint32_t job_flags;		/* --kill_invalid_dep, --gres-flags */
+	uint64_t job_flags;		/* --kill_invalid_dep, --gres-flags */
 	int threads_per_core;		/* --threads-per-core=n		*/
 	int ntasks_per_node;		/* --ntasks-per-node=n		*/
 	int ntasks_per_gpu;		/* --ntasks-per-gpu=n		*/
@@ -433,9 +436,28 @@ typedef struct {
 
 } slurm_opt_t;
 
+/*
+ * Parse a tres request and return the sum of the requested counts for a
+ * particular tres.
+ *
+ * IN in_val - The tres request of the format: "name[[:type]:count]". This can
+ *             be a comma-delimited list.
+ * IN tres_name - name of the tres to sum
+ * OUT cnt - total count
+ * IN/OUT save_ptr - pointer to make this function reentrant
+ * OUT rc - SLURM_SUCCESS if successful, an error code otherwise
+ *
+ * RET - true if there are additional values to parse, false if the whole string
+ *       has been parsed
+ */
+extern bool slurm_option_get_tres_per_tres(
+	char *in_val, char *tres_name, uint64_t *cnt, char **save_ptr, int *rc);
+
 extern struct option *slurm_option_table_create(slurm_opt_t *opt,
 						char **opt_string);
 extern void slurm_option_table_destroy(struct option *optz);
+extern void slurm_option_update_tres_per_task(int cnt, char *tres_str,
+					      char **tres_per_task_p);
 
 /*
  * Process individual argument for the current job component
@@ -497,7 +519,7 @@ extern bool slurm_option_set_by_env(slurm_opt_t *opt, int optval);
 /*
  * Was the option set by an data_t value?
  */
-extern bool slurm_option_set_by_env(slurm_opt_t *opt, int optval);
+extern bool slurm_option_set_by_data(slurm_opt_t *opt, int optval);
 
 /*
  * Get option value by common option name.
