@@ -40,6 +40,7 @@
 #include <sys/stat.h>
 
 #include "src/common/slurm_xlator.h"
+#include "src/common/fd.h"
 #include "src/common/strlcpy.h"
 
 #include "src/slurmctld/slurmctld.h"
@@ -123,6 +124,7 @@ extern int fini(void)
 	if (running_in_slurmctld()) {
 		switch_p_libstate_clear();
 		slingshot_fini_instant_on();
+		slingshot_fini_collectives();
 		slingshot_free_config();
 	} else
 		slingshot_free_services();
@@ -139,6 +141,7 @@ extern int switch_p_reconfig(void)
 {
 	if (running_in_slurmctld()) {
 		slingshot_fini_instant_on();
+		slingshot_fini_collectives();
 		if (!slingshot_setup_config(slurm_conf.switch_param))
 			return SLURM_ERROR;
 	}
@@ -152,7 +155,7 @@ extern int switch_p_reconfig(void)
 extern int switch_p_libstate_save(char *dir_name)
 {
 	int state_fd;
-	uint32_t actual_job_vnis;
+	uint32_t actual_job_vnis, actual_job_hwcoll;
 	buf_t *state_buf;
 	char *new_state_file, *state_file, *buf;
 	size_t buflen;
@@ -186,6 +189,20 @@ extern int switch_p_libstate_save(char *dir_name)
 			}
 		}
 	}
+	/* Pack only job_hwcoll slots that are being used */
+	actual_job_hwcoll = 0;
+	for (int i = 0; i < slingshot_state.num_job_hwcoll; i++) {
+		if (slingshot_state.job_hwcoll[i])
+			actual_job_hwcoll++;
+	}
+	pack32(actual_job_hwcoll, state_buf);
+	if (actual_job_hwcoll > 0) {
+		for (int i = 0; i < slingshot_state.num_job_hwcoll; i++) {
+			if (slingshot_state.job_hwcoll[i])
+				pack32(slingshot_state.job_hwcoll[i],
+				       state_buf);
+		}
+	}
 
 	/* Get file names for the current and new state files */
 	new_state_file = xstrdup(dir_name);
@@ -214,6 +231,10 @@ extern int switch_p_libstate_save(char *dir_name)
 		goto error;
 	}
 
+	if (fsync_and_close(state_fd, "switch"))
+		goto error;
+	state_fd = -1;
+
 	/* Overwrite the current state file with rename */
 	if (rename(new_state_file, state_file) == -1) {
 		error("Couldn't rename %s to %s: %m", new_state_file,
@@ -222,14 +243,14 @@ extern int switch_p_libstate_save(char *dir_name)
 	}
 
 	debug("State file %s saved", state_file);
-	close(state_fd);
 	FREE_NULL_BUFFER(state_buf);
 	xfree(new_state_file);
 	xfree(state_file);
 	return SLURM_SUCCESS;
 
 error:
-	close(state_fd);
+	if (state_fd)
+		close(state_fd);
 	FREE_NULL_BUFFER(state_buf);
 	unlink(new_state_file);
 	xfree(new_state_file);
@@ -272,12 +293,14 @@ extern int switch_p_libstate_restore(char *dir_name, bool recover)
 		goto error;
 	}
 
-	/* Validate version */
+	/* Validate version (support both version 1 and 2) */
 	safe_unpack32(&version, state_buf);
 	if (version != SLINGSHOT_STATE_VERSION) {
-		error("State file %s version %"PRIu32" != %d", state_file,
-			version, SLINGSHOT_STATE_VERSION);
-		goto error;
+		if (version != SLINGSHOT_STATE_VERSION_VER1) {
+			error("State file %s version %"PRIu32" != %d",
+				state_file, version, SLINGSHOT_STATE_VERSION);
+			goto error;
+		}
 	}
 
 	/* Unpack the rest into global state structure */
@@ -300,6 +323,22 @@ extern int switch_p_libstate_restore(char *dir_name, bool recover)
 				state_buf);
 		}
 	}
+	/* Unpack version 2 job_hwcoll state */
+	slingshot_state.num_job_hwcoll = 0;
+	slingshot_state.job_hwcoll = NULL;
+	if (version == SLINGSHOT_STATE_VERSION) {
+		safe_unpack32(&slingshot_state.num_job_hwcoll, state_buf);
+		if (slingshot_state.num_job_hwcoll > 0) {
+			debug("%s: unpacking %u job_hwcoll",
+			      state_file, slingshot_state.num_job_hwcoll);
+			slingshot_state.job_hwcoll = xcalloc(
+				slingshot_state.num_job_hwcoll,
+				sizeof(job_vni_t));
+			for (i = 0; i < slingshot_state.num_job_hwcoll; i++)
+				safe_unpack32(&slingshot_state.job_hwcoll[i],
+					state_buf);
+		}
+	}
 
 	debug("State file %s recovered", state_file);
 	FREE_NULL_BUFFER(state_buf);
@@ -345,19 +384,181 @@ extern int switch_p_alloc_jobinfo(switch_jobinfo_t **switch_job,
 	return SLURM_SUCCESS;
 }
 
+static void _copy_jobinfo(slingshot_jobinfo_t *old, slingshot_jobinfo_t *new)
+{
+	/* Copy static (non-malloced) fields */
+	memcpy(new, old, sizeof(slingshot_jobinfo_t));
+
+	/* Copy malloced fields */
+	if (old->num_vnis) {
+		size_t vnisz = old->num_vnis * sizeof(uint16_t);
+		new->vnis = xmalloc(vnisz);
+		memcpy(new->vnis, old->vnis, vnisz);
+	}
+
+	if (old->num_profiles) {
+		size_t profilesz = old->num_profiles *
+				   sizeof(slingshot_comm_profile_t);
+		new->profiles = xmalloc(profilesz);
+		memcpy(new->profiles, old->profiles, profilesz);
+	}
+
+	if (old->num_nics) {
+		size_t nicsz = old->num_nics * sizeof(slingshot_hsn_nic_t);
+		new->nics = xmalloc(nicsz);
+		memcpy(new->nics, old->nics, nicsz);
+	}
+
+	if (old->hwcoll) {
+		size_t hwcollsz = sizeof(slingshot_hwcoll_t);
+		new->hwcoll = xmalloc(hwcollsz);
+		memcpy(new->hwcoll, old->hwcoll, hwcollsz);
+		new->hwcoll->mcast_token = xstrdup(old->hwcoll->mcast_token);
+		new->hwcoll->fm_url = xstrdup(old->hwcoll->fm_url);
+	}
+}
+
+/*
+ * Get the slingshot jobinfo structure from a given step record.
+ */
+static slingshot_jobinfo_t *_get_slingshot_jobinfo(step_record_t *step_ptr)
+{
+	if (!step_ptr || !step_ptr->switch_job || !step_ptr->switch_job->data)
+		return NULL;
+	return (slingshot_jobinfo_t *) step_ptr->switch_job->data;
+}
+
+/*
+ * Copy slingshot jobinfo from the first het step in a non-het job
+ */
+static bool _copy_het_step_jobinfo(slingshot_jobinfo_t *jobinfo,
+				   step_record_t *step_ptr)
+{
+	slingshot_jobinfo_t *het_jobinfo;
+	step_record_t *het_step_ptr;
+	job_record_t *job_ptr;
+	slurm_step_id_t tmp_step_id;
+
+	/* For the first component, we build the jobinfo, not copy it */
+	if (step_ptr->step_id.step_het_comp == 0)
+		return false;
+
+	/* Get the step record for the first component */
+	job_ptr = step_ptr->job_ptr;
+	tmp_step_id.job_id = step_ptr->step_id.job_id,
+	tmp_step_id.step_id = step_ptr->step_id.step_id,
+	tmp_step_id.step_het_comp = 0,
+
+	het_step_ptr = find_step_record(job_ptr, &tmp_step_id);
+	het_jobinfo = _get_slingshot_jobinfo(het_step_ptr);
+
+	/* Copy it to the current step */
+	if (het_jobinfo) {
+		log_flag(SWITCH, "Copying slingshot jobinfo from %pS to %pS",
+			 het_step_ptr, step_ptr);
+		_copy_jobinfo(het_jobinfo, jobinfo);
+		return true;
+	}
+	return false;
+}
+
+/*
+ * Copy slingshot jobinfo from the first het step in a het job
+ */
+static bool _copy_het_job_jobinfo(slingshot_jobinfo_t *jobinfo,
+				  step_record_t *step_ptr)
+{
+	slingshot_jobinfo_t *het_jobinfo = NULL;
+	job_record_t *het_job_leader, *het_job_ptr, *job_ptr;
+	step_record_t *het_step_ptr;
+	list_itr_t *job_iter;
+
+	job_ptr = step_ptr->job_ptr;
+	if (!(het_job_leader = find_job_record(job_ptr->het_job_id)))
+		return false;
+
+	/* Loop through all heterogeneous job components */
+	job_iter = list_iterator_create(het_job_leader->het_job_list);
+	while ((het_job_ptr = list_next(job_iter))) {
+		slurm_step_id_t tmp_step_id = {
+			.job_id = het_job_ptr->job_id,
+			.step_id = step_ptr->step_id.step_id,
+			.step_het_comp = NO_VAL,
+		};
+
+		/*
+		 * If we get here without finding an existing jobinfo, we must
+		 * be the first component, so there's nothing to copy from.
+		 */
+		if (job_ptr->job_id == het_job_ptr->job_id) {
+			list_iterator_destroy(job_iter);
+			return false;
+		}
+
+		/* Look for a step in this job matching our step id */
+		het_step_ptr = find_step_record(het_job_ptr, &tmp_step_id);
+		het_jobinfo = _get_slingshot_jobinfo(het_step_ptr);
+
+		/* Copy it to the current step */
+		if (het_jobinfo) {
+			log_flag(SWITCH,
+				 "Copying slingshot jobinfo from %pS to %pS",
+				 het_step_ptr, step_ptr);
+			_copy_jobinfo(het_jobinfo, jobinfo);
+			list_iterator_destroy(job_iter);
+			return true;
+		}
+	}
+	list_iterator_destroy(job_iter);
+	return false;
+}
+
+/*
+ * Get the node count for a het step in a het job
+ */
+static uint32_t _get_het_job_node_cnt(step_record_t *step_ptr)
+{
+	hostlist_t *hl;
+	job_record_t *het_job_leader, *het_job_ptr, *job_ptr;
+	list_itr_t *job_iter;
+	uint32_t node_cnt;
+
+	job_ptr = step_ptr->job_ptr;
+	if (!(het_job_leader = find_job_record(job_ptr->het_job_id)))
+		return job_ptr->node_cnt;
+
+	/* Loop through all heterogeneous job components */
+	hl = hostlist_create(NULL);
+	job_iter = list_iterator_create(het_job_leader->het_job_list);
+	while ((het_job_ptr = list_next(job_iter)))
+		hostlist_push(hl, het_job_ptr->nodes);
+	list_iterator_destroy(job_iter);
+
+	/* Remove duplicates in the list */
+	hostlist_uniq(hl);
+	node_cnt = hostlist_count(hl);
+	hostlist_destroy(hl);
+	return node_cnt;
+}
+
 extern int switch_p_build_jobinfo(switch_jobinfo_t *switch_job,
 				  slurm_step_layout_t *step_layout,
 				  step_record_t *step_ptr)
 {
 	slingshot_jobinfo_t *job = (slingshot_jobinfo_t *) switch_job;
+	job_record_t *job_ptr;
+	uint32_t job_id;
+	uint32_t node_cnt;
+	char *node_list;
 
 	if (!step_ptr) {
 		fatal("switch_p_build_jobinfo: step_ptr NULL not supported");
 	}
-	xassert(step_ptr->job_ptr);
+	job_ptr = step_ptr->job_ptr;
+	xassert(job_ptr);
 	log_flag(SWITCH, "job_id=%u step_id=%u uid=%u network='%s'",
 		step_ptr->step_id.job_id, step_ptr->step_id.step_id,
-		step_ptr->job_ptr->user_id, step_ptr->network);
+		job_ptr->user_id, step_ptr->network);
 
 	if (!job) {
 		debug("switch_job was NULL");
@@ -365,20 +566,58 @@ extern int switch_p_build_jobinfo(switch_jobinfo_t *switch_job,
 	}
 	xassert(job->version == SLURM_PROTOCOL_VERSION);
 
+	/*
+	 * If this is a homogeneous step, or the first component in a
+	 * heterogeneous step, get the job ID, node list, and node count to use.
+	 *
+	 * Note that for heterogeneous steps, at the point this function is
+	 * called, the nodelist isn't available for all step components.
+	 * Without an accurate nodelist Instant On won't work, so we skip it.
+	 *
+	 * If this is not the first component in a heterogeneous step, copy the
+	 * jobinfo struct from the first component.
+	 */
+	if (job_ptr->het_job_id && (job_ptr->het_job_id != NO_VAL)) {
+		/* This is a het step in a het job */
+		if (_copy_het_job_jobinfo(job, step_ptr))
+			return SLURM_SUCCESS;
+
+		node_list = NULL;
+		node_cnt = _get_het_job_node_cnt(step_ptr);
+		job_id = job_ptr->het_job_id;
+	} else if (step_ptr->step_id.step_het_comp != NO_VAL) {
+		/* This is a het step in a non-het job */
+		if (_copy_het_step_jobinfo(job, step_ptr))
+			return SLURM_SUCCESS;
+
+		node_list = NULL;
+		node_cnt = job_ptr->node_cnt;
+		job_id = job_ptr->job_id;
+	} else {
+		/* This is a non-het step in a non-het job */
+		node_list = step_layout->node_list;
+		node_cnt = step_layout->node_cnt;
+		job_id = job_ptr->job_id;
+	}
+
 	/* Do VNI allocation/traffic classes/network limits */
-	if (!slingshot_setup_job_step(job, step_layout->node_cnt,
-				      step_ptr->step_id.job_id,
-				      step_ptr->network,
-				      step_ptr->job_ptr->network))
+	if (!slingshot_setup_job_step_vni(job, node_cnt, job_id,
+					  step_ptr->network, job_ptr->network))
+		return SLURM_ERROR;
+
+	/*
+	 * Reserve hardware collectives multicast addresses if configured
+	 */
+	if (!slingshot_setup_collectives(job, node_cnt, job_id,
+					 step_ptr->step_id.step_id))
 		return SLURM_ERROR;
 
 	/*
 	 * Fetch any Instant On data if configured;
 	 * don't fail launch on Instant On failure
 	 */
-	slingshot_fetch_instant_on(job, step_layout->node_list,
-				   step_layout->node_cnt);
-
+	if (node_list)
+		slingshot_fetch_instant_on(job, node_list, node_cnt);
 	return SLURM_SUCCESS;
 }
 
@@ -388,31 +627,7 @@ extern int switch_p_duplicate_jobinfo(switch_jobinfo_t *tmp,
 	slingshot_jobinfo_t *old = (slingshot_jobinfo_t *) tmp;
 	slingshot_jobinfo_t *new = xmalloc(sizeof(*new));
 
-	/* Copy static (non-malloced) fields */
-	memcpy(new, old, sizeof(slingshot_jobinfo_t));
-
-	/* Copy malloced fields */
-	if (old->num_vnis > 0) {
-		size_t vnisz = old->num_vnis * sizeof(uint16_t);
-		new->vnis = xmalloc(vnisz);
-		memcpy(new->vnis, old->vnis, vnisz);
-	}
-
-	if (old->num_profiles > 0) {
-		size_t profilesz = old->num_profiles *
-				   sizeof(slingshot_comm_profile_t);
-		new->profiles = xmalloc(profilesz);
-		memcpy(new->profiles, old->profiles, profilesz);
-	}
-
-	if (old->vni_pids)
-		new->vni_pids = bit_copy(old->vni_pids);
-
-	if (old->num_nics) {
-		size_t nicsz = old->num_nics * sizeof(slingshot_hsn_nic_t);
-		new->nics = xmalloc(nicsz);
-		memcpy(new->nics, old->nics, nicsz);
-	}
+	_copy_jobinfo(old, new);
 
 	*dest = (switch_jobinfo_t *)new;
 	return SLURM_SUCCESS;
@@ -424,9 +639,12 @@ extern void switch_p_free_jobinfo(switch_jobinfo_t *switch_job)
 	xassert(jobinfo);
 	xfree(jobinfo->vnis);
 	xfree(jobinfo->profiles);
-	if (jobinfo->vni_pids)
-		bit_free(jobinfo->vni_pids);
 	xfree(jobinfo->nics);
+	if (jobinfo->hwcoll) {
+		xfree(jobinfo->hwcoll->mcast_token);
+		xfree(jobinfo->hwcoll->fm_url);
+		xfree(jobinfo->hwcoll);
+	}
 	xfree(jobinfo);
 }
 
@@ -468,6 +686,22 @@ static void _pack_hsn_nic(slingshot_hsn_nic_t *nic, buf_t *buffer)
 	packstr(nic->device_name, buffer);
 }
 
+static void _pack_hwcoll(const slingshot_hwcoll_t *hwcoll, buf_t *buffer)
+{
+	/* Pack a boolean to let unpack routine know that hwcoll exists */
+	if (hwcoll) {
+		packbool(true, buffer);
+		pack32(hwcoll->job_id, buffer);
+		pack32(hwcoll->step_id, buffer);
+		packstr(hwcoll->mcast_token, buffer);
+		packstr(hwcoll->fm_url, buffer);
+		pack32(hwcoll->addrs_per_job, buffer);
+		pack32(hwcoll->num_nodes, buffer);
+	} else {
+		packbool(false, buffer);
+	}
+}
+
 static bool _unpack_comm_profile(slingshot_comm_profile_t *profile,
 				 buf_t *buffer)
 {
@@ -522,6 +756,41 @@ unpack_error:
 	return false;
 }
 
+static bool _unpack_hwcoll(slingshot_hwcoll_t **hwcollp, buf_t *buffer)
+{
+	slingshot_hwcoll_t *hwcoll = NULL;
+	bool got_hwcoll = false;
+	uint32_t name_len;
+
+	*hwcollp = NULL;
+	/* Unpack a boolean to see if hwcoll is packed in the buffer */
+	safe_unpackbool(&got_hwcoll, buffer);
+	if (got_hwcoll) {
+		hwcoll = xmalloc(sizeof(*hwcoll));
+
+		safe_unpack32(&hwcoll->job_id, buffer);
+		safe_unpack32(&hwcoll->step_id, buffer);
+
+		safe_unpackstr_xmalloc(&hwcoll->mcast_token, &name_len, buffer);
+		if (!hwcoll->mcast_token)
+			goto unpack_error;
+
+		safe_unpackstr_xmalloc(&hwcoll->fm_url, &name_len, buffer);
+		if (!hwcoll->fm_url)
+			goto unpack_error;
+
+		safe_unpack32(&hwcoll->addrs_per_job, buffer);
+		safe_unpack32(&hwcoll->num_nodes, buffer);
+
+		*hwcollp = hwcoll;
+	}
+
+	return true;
+
+unpack_error:
+	return false;
+}
+
 extern int switch_p_pack_jobinfo(switch_jobinfo_t *switch_job, buf_t *buffer,
 				 uint16_t protocol_version)
 {
@@ -530,7 +799,7 @@ extern int switch_p_pack_jobinfo(switch_jobinfo_t *switch_job, buf_t *buffer,
 
 	xassert(buffer);
 
-	if (protocol_version >= SLURM_23_02_PROTOCOL_VERSION) {
+	if (protocol_version >= SLURM_23_11_PROTOCOL_VERSION) {
 		/* nothing to pack, pack special "null" version number */
 		if (!jobinfo ||
 		    (jobinfo->version == SLINGSHOT_JOBINFO_NULL_VERSION)) {
@@ -555,13 +824,44 @@ extern int switch_p_pack_jobinfo(switch_jobinfo_t *switch_job, buf_t *buffer,
 		for (pidx = 0; pidx < jobinfo->num_profiles; pidx++) {
 			_pack_comm_profile(&jobinfo->profiles[pidx], buffer);
 		}
-		pack_bit_str_hex(jobinfo->vni_pids, buffer);
 		pack32(jobinfo->flags, buffer);
 		pack32(jobinfo->num_nics, buffer);
 		for (pidx = 0; pidx < jobinfo->num_nics; pidx++) {
 			_pack_hsn_nic(&jobinfo->nics[pidx], buffer);
 		}
-	} else if (protocol_version >= SLURM_22_05_PROTOCOL_VERSION) {
+		_pack_hwcoll(jobinfo->hwcoll, buffer);
+	} else if (protocol_version >= SLURM_23_02_PROTOCOL_VERSION) {
+		/* nothing to pack, pack special "null" version number */
+		if (!jobinfo ||
+		    (jobinfo->version == SLINGSHOT_JOBINFO_NULL_VERSION)) {
+			debug("Nothing to pack");
+			pack32(SLINGSHOT_JOBINFO_NULL_VERSION, buffer);
+			return SLURM_SUCCESS;
+		}
+
+		pack32(protocol_version, buffer);
+		pack16_array(jobinfo->vnis, jobinfo->num_vnis, buffer);
+		pack32(jobinfo->tcs, buffer);
+		_pack_slingshot_limits(&jobinfo->limits.txqs, buffer);
+		_pack_slingshot_limits(&jobinfo->limits.tgqs, buffer);
+		_pack_slingshot_limits(&jobinfo->limits.eqs, buffer);
+		_pack_slingshot_limits(&jobinfo->limits.cts, buffer);
+		_pack_slingshot_limits(&jobinfo->limits.tles, buffer);
+		_pack_slingshot_limits(&jobinfo->limits.ptes, buffer);
+		_pack_slingshot_limits(&jobinfo->limits.les, buffer);
+		_pack_slingshot_limits(&jobinfo->limits.acs, buffer);
+		pack32(jobinfo->depth, buffer);
+		pack32(jobinfo->num_profiles, buffer);
+		for (pidx = 0; pidx < jobinfo->num_profiles; pidx++) {
+			_pack_comm_profile(&jobinfo->profiles[pidx], buffer);
+		}
+		pack_bit_str_hex(NULL, buffer); /* formerly vni_pids, Unused */
+		pack32(jobinfo->flags, buffer);
+		pack32(jobinfo->num_nics, buffer);
+		for (pidx = 0; pidx < jobinfo->num_nics; pidx++) {
+			_pack_hsn_nic(&jobinfo->nics[pidx], buffer);
+		}
+	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		/* use SLURM_MIN_PROTOCOL_VERSION in 23.11 */
 		/* nothing to pack, pack special "null" version number */
 		if (!jobinfo ||
@@ -611,7 +911,7 @@ extern int switch_p_unpack_jobinfo(switch_jobinfo_t **switch_job, buf_t *buffer,
 	jobinfo = xmalloc(sizeof(*jobinfo));
 	*switch_job = (switch_jobinfo_t *)jobinfo;
 
-	if (protocol_version >= SLURM_23_02_PROTOCOL_VERSION) {
+	if (protocol_version >= SLURM_23_11_PROTOCOL_VERSION) {
 		safe_unpack32(&jobinfo->version, buffer);
 		if (jobinfo->version == SLINGSHOT_JOBINFO_NULL_VERSION) {
 			debug("Nothing to unpack");
@@ -644,7 +944,6 @@ extern int switch_p_unpack_jobinfo(switch_jobinfo_t **switch_job, buf_t *buffer,
 						  buffer))
 				goto unpack_error;
 		}
-		unpack_bit_str_hex(&jobinfo->vni_pids, buffer);
 		safe_unpack32(&jobinfo->flags, buffer);
 
 		safe_unpack32(&jobinfo->num_nics, buffer);
@@ -654,7 +953,55 @@ extern int switch_p_unpack_jobinfo(switch_jobinfo_t **switch_job, buf_t *buffer,
 			if (!_unpack_hsn_nic(&jobinfo->nics[pidx], buffer))
 				goto unpack_error;
 		}
-	} else if (protocol_version >= SLURM_22_05_PROTOCOL_VERSION) {
+		_unpack_hwcoll(&jobinfo->hwcoll, buffer);
+	} else if (protocol_version >= SLURM_23_02_PROTOCOL_VERSION) {
+		bitstr_t *vni_pids = NULL;
+		safe_unpack32(&jobinfo->version, buffer);
+		if (jobinfo->version == SLINGSHOT_JOBINFO_NULL_VERSION) {
+			debug("Nothing to unpack");
+			return SLURM_SUCCESS;
+		}
+		if (jobinfo->version != protocol_version) {
+			error("SLINGSHOT jobinfo version %"PRIu32" != %d",
+			      jobinfo->version, protocol_version);
+			goto error;
+		}
+
+		safe_unpack16_array(&jobinfo->vnis, &jobinfo->num_vnis, buffer);
+		safe_unpack32(&jobinfo->tcs, buffer);
+		if (!_unpack_slingshot_limits(&jobinfo->limits.txqs, buffer) ||
+		    !_unpack_slingshot_limits(&jobinfo->limits.tgqs, buffer) ||
+		    !_unpack_slingshot_limits(&jobinfo->limits.eqs, buffer) ||
+		    !_unpack_slingshot_limits(&jobinfo->limits.cts, buffer) ||
+		    !_unpack_slingshot_limits(&jobinfo->limits.tles, buffer) ||
+		    !_unpack_slingshot_limits(&jobinfo->limits.ptes, buffer) ||
+		    !_unpack_slingshot_limits(&jobinfo->limits.les, buffer) ||
+		    !_unpack_slingshot_limits(&jobinfo->limits.acs, buffer))
+			goto unpack_error;
+
+		safe_unpack32(&jobinfo->depth, buffer);
+		safe_unpack32(&jobinfo->num_profiles, buffer);
+		jobinfo->profiles = xcalloc(jobinfo->num_profiles,
+					    sizeof(slingshot_comm_profile_t));
+		for (pidx = 0; pidx < jobinfo->num_profiles; pidx++) {
+			if (!_unpack_comm_profile(&jobinfo->profiles[pidx],
+						  buffer))
+				goto unpack_error;
+		}
+		unpack_bit_str_hex(&vni_pids, buffer); /* Unused */
+		FREE_NULL_BITMAP(vni_pids);
+		safe_unpack32(&jobinfo->flags, buffer);
+
+		safe_unpack32(&jobinfo->num_nics, buffer);
+		jobinfo->nics = xcalloc(jobinfo->num_nics,
+					sizeof(slingshot_hsn_nic_t));
+		for (pidx = 0; pidx < jobinfo->num_nics; pidx++) {
+			if (!_unpack_hsn_nic(&jobinfo->nics[pidx], buffer))
+				goto unpack_error;
+		}
+		/* Not present in this version, set to none */
+		jobinfo->hwcoll = NULL;
+	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		/* use SLURM_MIN_PROTOCOL_VERSION in 23.11 */
 		safe_unpack32(&jobinfo->version, buffer);
 		if (jobinfo->version == SLINGSHOT_JOBINFO_NULL_VERSION) {
@@ -689,10 +1036,10 @@ extern int switch_p_unpack_jobinfo(switch_jobinfo_t **switch_job, buf_t *buffer,
 				goto unpack_error;
 		}
 		/* Not present in this version, set to none */
-		jobinfo->vni_pids = NULL;
 		jobinfo->flags = 0;
 		jobinfo->num_nics = 0;
 		jobinfo->nics = NULL;
+		jobinfo->hwcoll = NULL;
 	} else {
 		error("invalid protocol version");
 		goto error;
@@ -733,44 +1080,6 @@ extern int switch_p_job_init(stepd_step_rec_t *step)
 	return SLURM_SUCCESS;
 }
 
-extern int switch_p_job_suspend_test(switch_jobinfo_t *jobinfo)
-{
-	return SLURM_SUCCESS;
-}
-
-extern void switch_p_job_suspend_info_get(switch_jobinfo_t *jobinfo,
-					  void **suspend_info)
-{
-	return;
-}
-
-extern void switch_p_job_suspend_info_pack(void *suspend_info, buf_t *buffer,
-					   uint16_t protocol_version)
-{
-	return;
-}
-
-extern int switch_p_job_suspend_info_unpack(void **suspend_info, buf_t *buffer,
-					    uint16_t protocol_version)
-{
-	return SLURM_SUCCESS;
-}
-
-extern void switch_p_job_suspend_info_free(void *suspend_info)
-{
-	return;
-}
-
-extern int switch_p_job_suspend(void *suspend_info, int max_wait)
-{
-	return SLURM_SUCCESS;
-}
-
-extern int switch_p_job_resume(void *suspend_info, int max_wait)
-{
-	return SLURM_SUCCESS;
-}
-
 /*
  * Non-privileged
  */
@@ -820,7 +1129,6 @@ extern int switch_p_job_attach(switch_jobinfo_t *jobinfo, char ***env,
 	slingshot_jobinfo_t *job = (slingshot_jobinfo_t *)jobinfo;
 	int pidx, vidx;
 	char *svc_ids = NULL, *vnis = NULL, *devices = NULL, *tcss = NULL;
-	char *vni_pids = NULL;
 
 	if (job->num_profiles == 0)
 		return SLURM_SUCCESS;
@@ -840,39 +1148,25 @@ extern int switch_p_job_attach(switch_jobinfo_t *jobinfo, char ***env,
 	/* traffic classes are global */
 	xstrfmtcat(tcss, "0x%x", job->profiles[0].tcs);
 
-	/* convert bitstr_t into range list */
-	if (job->vni_pids)
-		vni_pids = bit_fmt_full(job->vni_pids);
-
-	log_flag(SWITCH, "%s=%s %s=%s %s=%s %s=%s %s=%s",
+	log_flag(SWITCH, "%s=%s %s=%s %s=%s %s=%s",
 		SLINGSHOT_SVC_IDS_ENV, svc_ids, SLINGSHOT_VNIS_ENV, vnis,
-		SLINGSHOT_DEVICES_ENV, devices, SLINGSHOT_TCS_ENV, tcss,
-		SLINGSHOT_INTER_VNI_PIDS_ENV, vni_pids);
+		SLINGSHOT_DEVICES_ENV, devices, SLINGSHOT_TCS_ENV, tcss);
 
 	env_array_overwrite(env, SLINGSHOT_SVC_IDS_ENV, svc_ids);
 	env_array_overwrite(env, SLINGSHOT_VNIS_ENV, vnis);
 	env_array_overwrite(env, SLINGSHOT_DEVICES_ENV, devices);
 	env_array_overwrite(env, SLINGSHOT_TCS_ENV, tcss);
-	if (vni_pids)
-		env_array_overwrite(env, SLINGSHOT_INTER_VNI_PIDS_ENV,
-				    vni_pids);
 
 	xfree(svc_ids);
 	xfree(vnis);
 	xfree(devices);
 	xfree(tcss);
-	xfree(vni_pids);
 
+	/* Add any collectives-related environment variables */
+	slingshot_collectives_env(job, env);
 	return SLURM_SUCCESS;
 }
 
-extern int switch_p_get_jobinfo(switch_jobinfo_t *switch_job, int key,
-				void *resulting_data)
-{
-	slurm_seterrno(EINVAL);
-	return SLURM_ERROR;
-}
-
 extern int switch_p_job_step_complete(switch_jobinfo_t *jobinfo, char *nodelist)
 {
 	slingshot_jobinfo_t *job = (slingshot_jobinfo_t *) jobinfo;
@@ -883,7 +1177,10 @@ extern int switch_p_job_step_complete(switch_jobinfo_t *jobinfo, char *nodelist)
 	 */
 	if (job) {
 		/* Free job step VNI */
-		slingshot_free_job_step(job);
+		slingshot_free_job_step_vni(job);
+
+		/* Release any hardware collectives multicast addresses */
+		slingshot_release_collectives_job_step(job);
 	}
 
 	return SLURM_SUCCESS;
@@ -915,10 +1212,17 @@ extern int switch_p_job_step_post_resume(stepd_step_rec_t *step)
 	return SLURM_SUCCESS;
 }
 
+/*
+ * Free any job VNIs, as well as any Slingshot hardware collectives
+ * multicast addresses associated with the job
+ */
 extern void switch_p_job_complete(uint32_t job_id)
 {
 	/* Free any job VNIs */
 	xassert(running_in_slurmctld());
 	log_flag(SWITCH, "switch_p_job_complete(%u)", job_id);
-	slingshot_free_job(job_id);
+	slingshot_free_job_vni(job_id);
+
+	/* Release any hardware collectives multicast addresses */
+	slingshot_release_collectives_job(job_id);
 }
