@@ -39,6 +39,7 @@
 #include "sacct.h"
 #include "src/common/cpu_frequency.h"
 #include "src/common/parse_time.h"
+#include "src/common/uid.h"
 #include "slurm/slurm.h"
 
 print_field_t *field = NULL;
@@ -283,7 +284,6 @@ extern void print_fields(type_t type, void *object)
 	slurmdb_step_rec_t *step = (slurmdb_step_rec_t *)object;
 	jobcomp_job_rec_t *job_comp = (jobcomp_job_rec_t *)object;
 	struct passwd *pw = NULL;
-	struct	group *gr = NULL;
 	int cpu_tres_rec_count = 0;
 	int step_cpu_tres_rec_count = 0;
 	char tmp1[128];
@@ -303,8 +303,7 @@ extern void print_fields(type_t type, void *object)
 	case JOB:
 		job_comp = NULL;
 		cpu_tres_rec_count = slurmdb_find_tres_count_in_string(
-			(job->tres_alloc_str && job->tres_alloc_str[0]) ?
-			job->tres_alloc_str : job->tres_req_str,
+			job->tres_alloc_str,
 			TRES_CPU);
 		break;
 	case JOBSTEP:
@@ -315,9 +314,7 @@ extern void print_fields(type_t type, void *object)
 			     step->tres_alloc_str, TRES_CPU)) == INFINITE64)
 			step_cpu_tres_rec_count =
 				slurmdb_find_tres_count_in_string(
-					(job->tres_alloc_str &&
-					 job->tres_alloc_str[0]) ?
-					job->tres_alloc_str : job->tres_req_str,
+					job->tres_alloc_str,
 					TRES_CPU);
 
 		job_comp = NULL;
@@ -910,23 +907,21 @@ extern void print_fields(type_t type, void *object)
 					     (curr_inx == field_count));
 			break;
 		case PRINT_GROUP:
+			tmp_char = NULL;
 			switch(type) {
 			case JOB:
-				tmp_uint32 = job->gid;
+				tmp_char = gid_to_string(job->gid);
 				break;
 			case JOBCOMP:
-				tmp_uint32 = job_comp->gid;
+				tmp_char = gid_to_string(job_comp->gid);
 				break;
 			default:
 				break;
 			}
-			tmp_char = NULL;
-			if ((gr=getgrgid(tmp_uint32)))
-				tmp_char=gr->gr_name;
-
 			field->print_routine(field,
 					     tmp_char,
 					     (curr_inx == field_count));
+			xfree(tmp_char);
 			break;
 		case PRINT_JOBID:
 			if (type == JOBSTEP)
@@ -1639,6 +1634,9 @@ extern void print_fields(type_t type, void *object)
 				break;
 			}
 
+			if (tmp_uint64 == INFINITE64)
+				tmp_uint64 = 0;
+
 			if (tmp_uint64 != NO_VAL64) {
 				convert_num_unit((double)tmp_uint64,
 						 outbuf, sizeof(outbuf),
@@ -1715,12 +1713,18 @@ extern void print_fields(type_t type, void *object)
 			/*
 			 * If eligible is 0 or -1, then the job was never
 			 * eligible to run, so planned time is 0.
+			 *
+			 * If the job hasn't started yet and has an end time,
+			 * presume the job was cancelled and use end time
+			 * instead of start time.
 			 */
 			switch(type) {
 			case JOB:
 				if (!job->eligible ||
 				    (job->eligible == INFINITE))
 					tmp_uint64 = 0;
+				else if ((job->start == NO_VAL) && job->end)
+					tmp_uint64 = job->end - job->eligible;
 				else if (job->start)
 					tmp_uint64 = job->start - job->eligible;
 				else
@@ -2033,12 +2037,16 @@ extern void print_fields(type_t type, void *object)
 					     (curr_inx == field_count));
 			break;
 		case PRINT_USER:
+		{
+			char *user = NULL;
 			switch(type) {
 			case JOB:
 				if (job->user)
 					tmp_char = job->user;
-				else if ((pw=getpwuid(job->uid)))
-						tmp_char = pw->pw_name;
+				else {
+					user = uid_to_string(job->uid);
+					tmp_char = user;
+				}
 				break;
 			case JOBCOMP:
 				tmp_char = job_comp->uid_name;
@@ -2050,7 +2058,9 @@ extern void print_fields(type_t type, void *object)
 			field->print_routine(field,
 					     tmp_char,
 					     (curr_inx == field_count));
+			xfree(user);
 			break;
+		}
 		case PRINT_USERCPU:
 			switch(type) {
 			case JOB:
