@@ -58,8 +58,8 @@
 #include "src/common/log.h"
 #include "src/common/proc_args.h"
 #include "src/common/read_config.h"
-#include "src/common/slurm_accounting_storage.h"
-#include "src/common/slurm_auth.h"
+#include "src/interfaces/accounting_storage.h"
+#include "src/interfaces/auth.h"
 #include "src/common/slurm_rlimits_info.h"
 #include "src/common/slurm_time.h"
 #include "src/common/uid.h"
@@ -67,6 +67,8 @@
 #include "src/common/xsignal.h"
 #include "src/common/xstring.h"
 
+#include "src/interfaces/hash.h"
+
 #include "src/slurmdbd/read_config.h"
 #include "src/slurmdbd/rpc_mgr.h"
 #include "src/slurmdbd/proc_req.h"
@@ -86,7 +88,8 @@ static int    dbd_sigarray[] = {	/* blocked signals for this process */
 	SIGUSR2, SIGTSTP, SIGXCPU, SIGQUIT,
 	SIGPIPE, SIGALRM, SIGABRT, SIGHUP, 0 };
 static int    debug_level = 0;		/* incremented for -v on command line */
-static int    foreground = 0;		/* run process as a daemon */
+static bool daemonize = true;		/* run process as a daemon */
+static int    setwd = 0;		/* change working directory -s  */
 static log_options_t log_opts = 	/* Log to stderr & syslog */
 	LOG_OPTS_INITIALIZER;
 static int	 new_nice = 0;
@@ -105,7 +108,6 @@ static void  _become_slurm_user(void);
 static void  _commit_handler_cancel(void);
 static void *_commit_handler(void *no_data);
 static void  _daemonize(void);
-static void  _default_sigaction(int sig);
 static void  _init_config(void);
 static void  _init_pidfile(void);
 static void  _kill_old_slurmdbd(void);
@@ -139,7 +141,7 @@ int main(int argc, char **argv)
 	_update_nice();
 
 	_kill_old_slurmdbd();
-	if (foreground == 0)
+	if (daemonize)
 		_daemonize();
 
 	/*
@@ -149,22 +151,32 @@ int main(int argc, char **argv)
 	 * able to write a core dump.
 	 */
 	_init_pidfile();
+	_become_slurm_user();
+
+	/*
+	 * This must happen before we spawn any threads
+	* which are not designed to handle them
+	*/
+	if (xsignal_block(dbd_sigarray) < 0)
+		error("Unable to block signals");
 
 	/*
 	 * Do plugin init's after _init_pidfile so systemd is happy as
-	 * slurm_acct_storage_init() could take a long time to finish if running
+	 * acct_storage_g_init() could take a long time to finish if running
 	 * for the first time after an upgrade.
 	 */
-	if (slurm_auth_init(NULL) != SLURM_SUCCESS) {
+	if (auth_g_init() != SLURM_SUCCESS) {
 		fatal("Unable to initialize authentication plugins");
 	}
-	if (slurm_acct_storage_init() != SLURM_SUCCESS) {
+	if (hash_g_init() != SLURM_SUCCESS) {
+		fatal("failed to initialize hash plugin");
+	}
+	if (acct_storage_g_init() != SLURM_SUCCESS) {
 		fatal("Unable to initialize %s accounting storage plugin",
 		      slurm_conf.accounting_storage_type);
 	}
 
-	_become_slurm_user();
-	if (foreground == 0)
+	if (daemonize || setwd)
 		_set_work_dir();
 	log_config();
 	init_dbd_stats();
@@ -174,9 +186,6 @@ int main(int argc, char **argv)
 		debug ("Unable to set dumpable to 1");
 #endif /* PR_SET_DUMPABLE */
 
-	if (xsignal_block(dbd_sigarray) < 0)
-		error("Unable to block signals");
-
 	/* Create attached thread for signal handling */
 	slurm_thread_create(&signal_handler_thread, _signal_handler, NULL);
 
@@ -188,12 +197,12 @@ int main(int argc, char **argv)
 
 	/*
 	 * If we are tracking wckey we need to cache wckeys,
-	 * if we aren't only cache the users, qos, and tres.
+	 * if we aren't only cache the assoc, users, qos, and tres.
 	 */
 	assoc_init_arg.cache_level = ASSOC_MGR_CACHE_USER |
-		ASSOC_MGR_CACHE_QOS | ASSOC_MGR_CACHE_TRES;
-	if (slurmdbd_conf->track_wckey)
-		assoc_init_arg.cache_level |= ASSOC_MGR_CACHE_WCKEY;
+		ASSOC_MGR_CACHE_ASSOC |
+		ASSOC_MGR_CACHE_QOS | ASSOC_MGR_CACHE_TRES |
+		ASSOC_MGR_CACHE_WCKEY;
 
 	db_conn = acct_storage_g_get_connection(0, NULL, true, NULL);
 	if (assoc_mgr_init(db_conn, &assoc_init_arg, errno) == SLURM_ERROR) {
@@ -316,8 +325,9 @@ end_it:
 	}
 
 	assoc_mgr_fini(0);
-	slurm_acct_storage_fini();
-	slurm_auth_fini();
+	acct_storage_g_fini();
+	auth_g_fini();
+	hash_g_fini();
 	log_fini();
 	free_slurmdbd_conf();
 	slurm_mutex_lock(&rpc_mutex);
@@ -430,7 +440,7 @@ static void  _init_config(void)
 {
 	struct rlimit rlim;
 
-	rlimits_maximize_nofile();
+	rlimits_use_max_nofile();
 	if (getrlimit(RLIMIT_CORE, &rlim) == 0) {
 		rlim.rlim_cur = rlim.rlim_max;
 		(void) setrlimit(RLIMIT_CORE, &rlim);
@@ -461,10 +471,10 @@ static void _parse_commandline(int argc, char **argv)
 	char *tmp_char;
 
 	opterr = 0;
-	while ((c = getopt(argc, argv, "Dhn:R::vV")) != -1)
+	while ((c = getopt(argc, argv, "Dhn:R::svV")) != -1)
 		switch (c) {
 		case 'D':
-			foreground = 1;
+			daemonize = 0;
 			break;
 		case 'h':
 			_usage(argv[0]);
@@ -487,6 +497,9 @@ static void _parse_commandline(int argc, char **argv)
 				slurm_addto_char_list(lft_rgt_list, optarg);
 			}
 			break;
+		case 's':
+			setwd = 1;
+			break;
 		case 'v':
 			debug_level++;
 			break;
@@ -517,6 +530,8 @@ static void _usage(char *prog_name)
 		"\n\t\tLft and rgt values are used to distinguish "
 		"\n\t\thierarical groups in the slurm accounting database.  "
 		"\n\t\tThis option should be very rarely used.\n");
+	fprintf(stderr, "  -s         \t"
+		"Change working directory to LogFile dirname or /var/tmp/.\n");
 	fprintf(stderr, "  -v         \t"
 		"Verbose mode. Multiple -v's increase verbosity.\n");
 	fprintf(stderr, "  -V         \t"
@@ -535,14 +550,14 @@ static void _update_logging(bool startup)
 
 	log_opts.logfile_level = slurmdbd_conf->debug_level;
 
-	if (foreground)
+	if (!daemonize)
 		log_opts.stderr_level  = slurmdbd_conf->debug_level;
 	else
 		log_opts.stderr_level = LOG_LEVEL_QUIET;
 
 	if (slurmdbd_conf->syslog_debug != LOG_LEVEL_END) {
 		log_opts.syslog_level =	slurmdbd_conf->syslog_debug;
-	} else if (foreground) {
+	} else if (!daemonize) {
 		log_opts.syslog_level = LOG_LEVEL_QUIET;
 	} else if ((slurmdbd_conf->debug_level > LOG_LEVEL_QUIET)
 		   && !slurmdbd_conf->log_file) {
@@ -845,6 +860,8 @@ static int _send_slurmctld_register_req(slurmdb_cluster_rec_t *cluster_rec)
 		       cluster_rec->control_host);
 	fd = slurm_open_msg_conn(&ctld_address);
 	if (fd < 0) {
+		log_flag(NET, "%s: slurm_open_msg_conn(%pA): %m",
+			 __func__, &ctld_address);
 		rc = SLURM_ERROR;
 	} else {
 		slurm_msg_t out_msg;
@@ -855,7 +872,7 @@ static int _send_slurmctld_register_req(slurmdb_cluster_rec_t *cluster_rec)
 		out_msg.protocol_version = cluster_rec->rpc_version;
 		slurm_send_node_msg(fd, &out_msg);
 		/* We probably need to add matching recv_msg function
-		 * for an arbitray fd or should these be fire
+		 * for an arbitrary fd or should these be fire
 		 * and forget?  For this, that we can probably
 		 * forget about it */
 		close(fd);
@@ -874,11 +891,11 @@ static void *_signal_handler(void *no_data)
 	(void) pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
 
 	/* Make sure no required signals are ignored (possibly inherited) */
-	_default_sigaction(SIGINT);
-	_default_sigaction(SIGTERM);
-	_default_sigaction(SIGHUP);
-	_default_sigaction(SIGABRT);
-	_default_sigaction(SIGUSR2);
+	xsignal_default(SIGINT);
+	xsignal_default(SIGTERM);
+	xsignal_default(SIGHUP);
+	xsignal_default(SIGABRT);
+	xsignal_default(SIGUSR2);
 
 	while (1) {
 		xsignal_sigset_create(sig_array, &set);
@@ -911,24 +928,6 @@ static void *_signal_handler(void *no_data)
 
 }
 
-/* Reset some signals to their default state to clear any
- * inherited signal states */
-static void _default_sigaction(int sig)
-{
-	struct sigaction act;
-
-	if (sigaction(sig, NULL, &act)) {
-		error("sigaction(%d): %m", sig);
-		return;
-	}
-	if (act.sa_handler != SIG_IGN)
-		return;
-
-	act.sa_handler = SIG_DFL;
-	if (sigaction(sig, &act, NULL))
-		error("sigaction(%d): %m", sig);
-}
-
 static void _become_slurm_user(void)
 {
 	gid_t slurm_user_gid;
@@ -960,7 +959,7 @@ static void _become_slurm_user(void)
 	/* Set GID to GID of SlurmUser */
 	if ((slurm_user_gid != getegid()) &&
 	    (setgid(slurm_user_gid))) {
-		fatal("Failed to set GID to %d", slurm_user_gid);
+		fatal("Failed to set GID to %u", slurm_user_gid);
 	}
 
 	/* Set UID to UID of SlurmUser */
@@ -971,7 +970,7 @@ static void _become_slurm_user(void)
 	}
 }
 
-extern void _restart_self(int argc, char **argv)
+static void _restart_self(int argc, char **argv)
 {
 	info("Restarting self");
 	if (execvp(argv[0], argv))
