@@ -41,11 +41,11 @@
 
 #include <fcntl.h>
 #include <grp.h>
+#include <limits.h>
 #include <netdb.h>
 #include <signal.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/param.h>           /* MAXPATHLEN */
 #include <sys/resource.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -109,7 +109,7 @@ typedef struct allocation_info {
 typedef struct het_job_resp_struct {
 	char **alias_list;
 	uint16_t *cpu_cnt;
-	hostlist_t host_list;
+	hostlist_t *host_list;
 	uint32_t node_cnt;
 } het_job_resp_struct_t;
 
@@ -124,10 +124,8 @@ extern char **environ;
  * Prototypes:
  */
 
-static int  _become_user(void);
 static void _call_spank_fini(void);
 static int  _call_spank_local_user(srun_job_t *job, slurm_opt_t *opt_local);
-static void _default_sigaction(int sig);
 static long _diff_tv_str(struct timeval *tv1, struct timeval *tv2);
 static void _handle_intr(srun_job_t *job);
 static void _handle_pipe(void);
@@ -143,7 +141,6 @@ static void _set_env_vars(resource_allocation_response_msg_t *resp,
 static void _set_env_vars2(resource_allocation_response_msg_t *resp,
 			   int het_job_offset);
 static void _set_ntasks(allocation_info_t *ai, slurm_opt_t *opt_local);
-static void _set_prio_process_env(void);
 static int  _set_rlimit_env(void);
 static void _set_submit_dir_env(void);
 static int  _set_umask_env(void);
@@ -168,7 +165,7 @@ job_create_noalloc(void)
 	uint16_t cpn[1];
 	uint32_t cpu_count_reps[1];
 	slurm_opt_t *opt_local = &opt;
-	hostlist_t  hl = hostlist_create(opt_local->nodelist);
+	hostlist_t *hl = hostlist_create(opt_local->nodelist);
 
 	if (!hl) {
 		error("Invalid node list `%s' specified", opt_local->nodelist);
@@ -281,7 +278,7 @@ extern srun_job_t *job_step_create_allocation(
 	uint32_t job_id = resp->job_id;
 	srun_job_t *job = NULL;
 	allocation_info_t *ai = xmalloc(sizeof(allocation_info_t));
-	hostlist_t hl = NULL;
+	hostlist_t *hl = NULL;
 	char *buf = NULL;
 	int count = 0;
 	uint32_t alloc_count = 0;
@@ -305,8 +302,8 @@ extern srun_job_t *job_step_create_allocation(
 
 	/* exclude is handled elsewhere for het steps */
 	if (!local_het_step && opt_local->exclude) {
-		hostlist_t exc_hl = hostlist_create(opt_local->exclude);
-		hostlist_t inc_hl = NULL;
+		hostlist_t *exc_hl = hostlist_create(opt_local->exclude);
+		hostlist_t *inc_hl = NULL;
 		char *node_name = NULL;
 
 		hl = hostlist_create(ai->nodelist);
@@ -372,7 +369,7 @@ extern srun_job_t *job_step_create_allocation(
 			if (count < ai->nnodes) {
 				/* add more nodes to get correct number for
 				   allocation */
-				hostlist_t tmp_hl = hostlist_copy(hl);
+				hostlist_t *tmp_hl = hostlist_copy(hl);
 				int i = 0;
 				int diff = ai->nnodes - count;
 				buf = hostlist_ranged_string_xmalloc(inc_hl);
@@ -732,6 +729,13 @@ extern void init_srun(int argc, char **argv, log_options_t *logopt,
 			het_job_fini = true;
 		}
 	}
+
+	if (!mpi_g_client_init(&sropt.mpi_type)) {
+		error("Invalid MPI type '%s', --mpi=list for acceptable types",
+		      sropt.mpi_type);
+		exit(error_exit);
+	}
+
 	_post_opts(opt_list);
 
 	/*
@@ -750,7 +754,7 @@ extern void init_srun(int argc, char **argv, log_options_t *logopt,
 	}
 
 	(void) _set_rlimit_env();
-	_set_prio_process_env();
+	set_prio_process_env();
 	(void) _set_umask_env();
 	_set_submit_dir_env();
 
@@ -782,9 +786,9 @@ static void _set_step_opts(slurm_opt_t *opt_local)
 }
 
 static int _handle_het_step_exclude(srun_job_t *job, slurm_opt_t *opt_local,
-				    hostlist_t exclude_hl_in)
+				    hostlist_t *exclude_hl_in)
 {
-	hostlist_t exclude_hl, allocation_hl;
+	hostlist_t *exclude_hl, *allocation_hl;
 	int rc = SLURM_SUCCESS;
 
 	if (!exclude_hl_in || !hostlist_count(exclude_hl_in))
@@ -812,7 +816,7 @@ static int _handle_het_step_exclude(srun_job_t *job, slurm_opt_t *opt_local,
 
 	if (opt_local->nodelist) {
 		char *node_name = NULL;
-		hostlist_t inc_hl = hostlist_create(opt_local->nodelist);
+		hostlist_t *inc_hl = hostlist_create(opt_local->nodelist);
 		while ((node_name = hostlist_shift(exclude_hl))) {
 			if (hostlist_find(inc_hl, node_name) >= 0) {
 				error("Requested nodelist %s overlaps with excluded %s.",
@@ -854,7 +858,7 @@ static int _create_job_step(srun_job_t *job, bool use_all_cpus,
 	int rc = 0;
 
 	if (srun_job_list) {
-		hostlist_t exclude_hl = NULL;
+		hostlist_t *exclude_hl = NULL;
 
 		if (local_het_step)
 			exclude_hl = hostlist_create(NULL);
@@ -948,7 +952,7 @@ static int _create_job_step(srun_job_t *job, bool use_all_cpus,
 			 * Merge numeric values into single range
 			 * (e.g. "10-12,13-15,16-18" -> "10-18")
 			 */
-			hostset_t hs;
+			hostset_t *hs;
 			char *tmp = NULL, *sep;
 			xstrfmtcat(tmp, "[%s]", resv_ports);
 			hs = hostset_create(tmp);
@@ -1042,7 +1046,7 @@ static char *_compress_het_job_nodelist(List used_resp_list)
 	ListIterator resp_iter;
 	char *aliases = NULL, *save_ptr = NULL, *tok, *tmp;
 	char *het_job_nodelist = NULL, *node_name;
-	hostset_t hs;
+	hostset_t *hs;
 	int cnt, i, j, k;
 	uint16_t *cpus;
 	uint32_t *reps, cpu_inx;
@@ -1148,14 +1152,6 @@ static char *_compress_het_job_nodelist(List used_resp_list)
 	}
 	xfree(tmp);
 
-	if (aliases) {
-		if (setenv("SLURM_NODE_ALIASES", aliases, 1) < 0) {
-			error("%s: Unable to set SLURM_NODE_ALIASES in environment",
-			      __func__);
-		}
-		xfree(aliases);
-	}
-
 	xfree(reps);
 	xfree(cpus);
 	hostset_destroy(hs);
@@ -1181,9 +1177,25 @@ static void _copy_job_resp(List job_resp_list, int count)
 		list_append(job_resp_list, new);
 	}
 }
+static void _check_gpus_per_socket(slurm_opt_t *opt_local)
+{
+	static bool checked = false; /* Only log the warning once */
 
-extern void create_srun_job(void **p_job, bool *got_alloc,
-			    bool slurm_started, bool handle_signals)
+	if (!opt_local->gpus_per_socket || checked)
+		return;
+
+	checked = true;
+	if (opt_local->gpus_per_socket &&
+	    !slurm_option_set_by_env(opt_local, LONG_OPT_GPUS_PER_SOCKET)) {
+		/*
+		 * gpus_per_socket does not work for steps.
+		 * If it is set by env, it was likely inherited by the job.
+		 */
+		warning("Ignoring --gpus-per-socket because it can only be specified at job allocation time, not during step allocation.");
+	}
+}
+
+extern void create_srun_job(void **p_job, bool *got_alloc)
 {
 	resource_allocation_response_msg_t *resp;
 	List job_resp_list = NULL, srun_job_list = NULL;
@@ -1324,6 +1336,7 @@ extern void create_srun_job(void **p_job, bool *got_alloc,
 							opt_local->min_nodes;
 					}
 				}
+				_check_gpus_per_socket(opt_local);
 				if (!tres_license_error_logged &&
 				    !slurm_option_set_by_env(
 					    opt_local,
@@ -1447,7 +1460,7 @@ extern void create_srun_job(void **p_job, bool *got_alloc,
 			setenvfs("SLURM_JOB_NAME=%s", opt.argv[0]);
 
 		if (opt_list) {
-			job_resp_list = allocate_het_job_nodes(handle_signals);
+			job_resp_list = allocate_het_job_nodes();
 			if (!job_resp_list)
 				exit(error_exit);
 			srun_job_list = list_create(NULL);
@@ -1485,7 +1498,7 @@ extern void create_srun_job(void **p_job, bool *got_alloc,
 					 het_job_offset + 1);
 			}
 		} else {
-			if (!(resp = allocate_nodes(handle_signals, &opt)))
+			if (!(resp = allocate_nodes(&opt)))
 				exit(error_exit);
 			*got_alloc = true;
 			my_job_id = resp->job_id;
@@ -1507,11 +1520,6 @@ extern void create_srun_job(void **p_job, bool *got_alloc,
 				_compress_het_job_nodelist(job_resp_list);
 		}
 
-		/*
-		 *  Become --uid user
-		 */
-		if (_become_user () < 0)
-			fatal("Unable to assume uid=%u", opt.uid);
 		if (_create_job_step(job, true, srun_job_list, het_job_id,
 				     het_job_nodelist) < 0) {
 			slurm_complete_job(my_job_id, 1);
@@ -1532,18 +1540,10 @@ extern void create_srun_job(void **p_job, bool *got_alloc,
 	}
 
 	/*
-	 *  Become --uid user
+	 * Spawn process to ensure clean-up of job and/or step
+	 * on abnormal termination
 	 */
-	if (_become_user () < 0)
-		fatal("Unable to assume uid=%u", opt.uid);
-
-	if (!slurm_started) {
-		/*
-		 * Spawn process to ensure clean-up of job and/or step
-		 * on abnormal termination
-		 */
-		shepherd_fd = _shepherd_spawn(job, srun_job_list, *got_alloc);
-	}
+	shepherd_fd = _shepherd_spawn(job, srun_job_list, *got_alloc);
 
 	if (opt_list)
 		*p_job = (void *) srun_job_list;
@@ -1554,16 +1554,10 @@ extern void create_srun_job(void **p_job, bool *got_alloc,
 	        _srun_cli_filter_post_submit(my_job_id, job->step_id.step_id);
 }
 
-extern void pre_launch_srun_job(srun_job_t *job, bool slurm_started,
-				bool handle_signals, slurm_opt_t *opt_local)
+extern void pre_launch_srun_job(srun_job_t *job, slurm_opt_t *opt_local)
 {
-	if (handle_signals && !signal_thread) {
+	if (!signal_thread)
 		slurm_thread_create(&signal_thread, _srun_signal_mgr, job);
-	}
-
-	/* if running from poe This already happened in srun. */
-	if (slurm_started)
-		return;
 
 	_run_srun_prolog(job);
 	if (_call_spank_local_user(job, opt_local) < 0) {
@@ -1575,12 +1569,8 @@ extern void pre_launch_srun_job(srun_job_t *job, bool slurm_started,
 	env_array_merge(&job->env, (const char **)environ);
 }
 
-extern void fini_srun(srun_job_t *job, bool got_alloc, uint32_t *global_rc,
-		      bool slurm_started)
+extern void fini_srun(srun_job_t *job, bool got_alloc, uint32_t *global_rc)
 {
-	/* If running from poe, most of this already happened in srun. */
-	if (slurm_started)
-		goto cleanup;
 	if (got_alloc) {
 		cleanup_allocation();
 
@@ -1592,15 +1582,13 @@ extern void fini_srun(srun_job_t *job, bool got_alloc, uint32_t *global_rc,
 	}
 	_shepherd_notify(shepherd_fd);
 
-cleanup:
 	if (signal_thread) {
 		srun_shutdown = true;
 		pthread_kill(signal_thread, SIGINT);
 		pthread_join(signal_thread,  NULL);
 	}
 
-	if (!slurm_started)
-		_run_srun_epilog(job);
+	_run_srun_epilog(job);
 
 	step_ctx_destroy(job->step_ctx);
 
@@ -1655,7 +1643,8 @@ job_force_termination(srun_job_t *job)
 		}
 		if (kill_sent == 1) {
 			/* Try sending SIGKILL through slurmctld */
-			slurm_kill_job_step(job->step_id.job_id, job->step_id.step_id, SIGKILL);
+			slurm_kill_job_step(job->step_id.job_id,
+					    job->step_id.step_id, SIGKILL, 0);
 		}
 	}
 	kill_sent++;
@@ -1778,7 +1767,7 @@ static char *
 _normalize_hostlist(const char *hostlist)
 {
 	char *buf = NULL;
-	hostlist_t hl = hostlist_create(hostlist);
+	hostlist_t *hl = hostlist_create(hostlist);
 
 	if (hl)	{
 		buf = hostlist_ranged_string_xmalloc(hl);
@@ -1790,35 +1779,6 @@ _normalize_hostlist(const char *hostlist)
 	return buf;
 }
 
-static int _become_user (void)
-{
-	char *user;
-
-	/* Already the user, so there's nothing to change. Return early. */
-	if (opt.uid == SLURM_AUTH_NOBODY)
-		return SLURM_SUCCESS;
-
-	if (!(user = uid_to_string_or_null(opt.uid))) {
-		xfree(user);
-		return (error ("Invalid user id %u: %m", opt.uid));
-	}
-
-	if ((opt.gid != getgid()) && (setgid(opt.gid) < 0)) {
-		xfree(user);
-		return (error ("setgid: %m"));
-	}
-
-	if (initgroups(user, gid_from_uid(opt.uid)))
-		return (error ("initgroups: %m"));
-
-	xfree(user);
-
-	if (setuid (opt.uid) < 0)
-		return (error ("setuid: %m"));
-
-	return (0);
-}
-
 static int _call_spank_local_user(srun_job_t *job, slurm_opt_t *opt_local)
 {
 	struct spank_launcher_job_info info[1];
@@ -1834,21 +1794,6 @@ static int _call_spank_local_user(srun_job_t *job, slurm_opt_t *opt_local)
 	return spank_local_user(info);
 }
 
-static void _default_sigaction(int sig)
-{
-	struct sigaction act;
-	if (sigaction(sig, NULL, &act)) {
-		error("sigaction(%d): %m", sig);
-		return;
-	}
-	if (act.sa_handler != SIG_IGN)
-		return;
-
-	act.sa_handler = SIG_DFL;
-	if (sigaction(sig, &act, NULL))
-		error("sigaction(%d): %m", sig);
-}
-
 /* Return the number of microseconds between tv1 and tv2 with a maximum
  * a maximum value of 10,000,000 to prevent overflows */
 static long _diff_tv_str(struct timeval *tv1, struct timeval *tv2)
@@ -2034,15 +1979,6 @@ static void _set_env_vars(resource_allocation_response_msg_t *resp,
 	}
 	xfree(key);
 
-	key = _build_key("SLURM_NODE_ALIASES", het_job_offset);
-	if (resp->alias_list) {
-		if (setenv(key, resp->alias_list, 1) < 0)
-			error("unable to set %s in environment", key);
-	} else {
-		unsetenv(key);
-	}
-	xfree(key);
-
 	if (resp->env_size) {	/* Used to set Burst Buffer environment */
 		for (i = 0; i < resp->env_size; i++) {
 			tmp = xstrdup(resp->environment[i]);
@@ -2133,43 +2069,6 @@ static void _set_env_vars2(resource_allocation_response_msg_t *resp,
 		}
 		xfree(key);
 	}
-
-	if (resp->alias_list) {
-		key = _build_key("SLURM_NODE_ALIASES", het_job_offset);
-		if (!getenv(key) &&
-		    (setenvf(NULL, key, "%s", resp->alias_list) < 0)) {
-			error("unable to set %s in environment", key);
-		}
-		xfree(key);
-	}
-}
-
-/*
- * _set_prio_process_env
- *
- * Set the internal SLURM_PRIO_PROCESS environment variable to support
- * the propagation of the users nice value and the "PropagatePrioProcess"
- * config keyword.
- */
-static void  _set_prio_process_env(void)
-{
-	int retval;
-
-	errno = 0; /* needed to detect a real failure since prio can be -1 */
-
-	if ((retval = getpriority (PRIO_PROCESS, 0)) == -1)  {
-		if (errno) {
-			error ("getpriority(PRIO_PROCESS): %m");
-			return;
-		}
-	}
-
-	if (setenvf (NULL, "SLURM_PRIO_PROCESS", "%d", retval) < 0) {
-		error ("unable to set SLURM_PRIO_PROCESS in environment");
-		return;
-	}
-
-	debug ("propagating SLURM_PRIO_PROCESS=%d", retval);
 }
 
 /* Set SLURM_RLIMIT_* environment variables with current resource
@@ -2231,7 +2130,7 @@ static int _set_rlimit_env(void)
  * variables within current state */
 static void _set_submit_dir_env(void)
 {
-	char buf[MAXPATHLEN + 1], host[256];
+	char buf[PATH_MAX], host[256];
 
 	/* Only set these environment variables in new allocations */
 	if (sropt.jobid != NO_VAL)
@@ -2241,7 +2140,7 @@ static void _set_submit_dir_env(void)
 		    slurm_conf.cluster_name) < 0)
 		error("unable to set SLURM_CLUSTER_NAME in environment");
 
-	if ((getcwd(buf, MAXPATHLEN)) == NULL)
+	if ((getcwd(buf, PATH_MAX)) == NULL)
 		error("getcwd failed: %m");
 	else if (setenvf(NULL, "SLURM_SUBMIT_DIR", "%s", buf) < 0)
 		error("unable to set SLURM_SUBMIT_DIR in environment");
@@ -2339,13 +2238,14 @@ static int _shepherd_spawn(srun_job_t *job, List srun_job_list, bool got_alloc)
 		job_iter  = list_iterator_create(srun_job_list);
 		while ((job = list_next(job_iter))) {
 			(void) slurm_kill_job_step(job->step_id.job_id, job->step_id.step_id,
-						   SIGKILL);
+						   SIGKILL, 0);
 			if (got_alloc)
 				slurm_complete_job(job->step_id.job_id, NO_VAL);
 		}
 		list_iterator_destroy(job_iter);
 	} else {
-		(void) slurm_kill_job_step(job->step_id.job_id, job->step_id.step_id, SIGKILL);
+		(void) slurm_kill_job_step(job->step_id.job_id,
+					   job->step_id.step_id, SIGKILL, 0);
 		if (got_alloc)
 			slurm_complete_job(job->step_id.job_id, NO_VAL);
 	}
@@ -2364,7 +2264,7 @@ static void *_srun_signal_mgr(void *job_ptr)
 
 	/* Make sure no required signals are ignored (possibly inherited) */
 	for (i = 0; sig_array[i]; i++)
-		_default_sigaction(sig_array[i]);
+		xsignal_default(sig_array[i]);
 	while (!srun_shutdown) {
 		xsignal_sigset_create(sig_array, &set);
 		rc = sigwait(&set, &sig);
